@@ -28,7 +28,7 @@ use std::{
 
 use debugserver_types::*;
 use gazebo::prelude::*;
-pub use library::*;
+pub(crate) use library::*;
 use serde_json::{Map, Value};
 use starlark::{
     codemap::{FileSpan, FileSpanRef},
@@ -108,7 +108,7 @@ impl Backend {
                 } else {
                     let breaks = breakpoints.lock().unwrap();
                     breaks
-                        .get(span_loc.file().filename())
+                        .get(span_loc.filename())
                         .map(|set| set.contains(&span_loc.to_file_span()))
                         .unwrap_or_default()
                 };
@@ -281,7 +281,7 @@ impl DebugServer for Backend {
                 s.end_line = Some(span.end_line as i64 + 1);
                 s.end_column = Some(span.end_column as i64 + 1);
                 s.source = Some(Source {
-                    path: Some(loc.file().filename().to_owned()),
+                    path: Some(loc.filename().to_owned()),
                     ..Source::default()
                 })
             }
@@ -292,7 +292,7 @@ impl DebugServer for Backend {
         // We record the location of the call, but DAP wants the location we are at.
         // We also have them in the wrong order
         self.with_ctx(box |span, eval| {
-            let frames = eval.call_stack();
+            let frames = eval.call_stack().into_frames();
             let mut next = Some(span.to_file_span());
             let mut res = Vec::with_capacity(frames.len() + 1);
             for (i, x) in frames.iter().rev().enumerate() {
@@ -377,7 +377,7 @@ impl DebugServer for Backend {
     }
 }
 
-pub fn server() {
+pub(crate) fn server() {
     let (sender, receiver) = channel();
     DapService::run(|client| Backend {
         client,
