@@ -35,7 +35,10 @@ use starlark::read_line::ReadLine;
 use structopt::{clap::AppSettings, StructOpt};
 use walkdir::WalkDir;
 
-use crate::types::{LintMessage, Message, Severity};
+use crate::{
+    eval::ContextMode,
+    types::{LintMessage, Message, Severity},
+};
 
 mod dap;
 mod eval;
@@ -48,7 +51,7 @@ mod types;
     about = "Evaluate Starlark code",
     global_settings(&[AppSettings::ColoredHelp]),
 )]
-pub struct Args {
+struct Args {
     #[structopt(
         long = "interactive",
         long = "repl",
@@ -63,21 +66,19 @@ pub struct Args {
     #[structopt(long = "dap", help = "Start a DAP server.")]
     dap: bool,
 
-    #[structopt(long = "check", help = "Run checks and lints.")]
+    #[structopt(
+        long = "check",
+        help = "Run checks and lints.",
+        conflicts_with_all = &["lsp", "dap"],
+    )]
     check: bool,
 
-    #[structopt(long = "info", help = "Show information about the code.")]
-    info: bool,
-
-    #[structopt(long = "json", help = "Show output as JSON lines.")]
-    json: bool,
-
     #[structopt(
-        long = "repeat",
-        help = "Number of times to repeat the execution",
-        default_value = "1"
+        long = "json",
+        help = "Show output as JSON lines.",
+        conflicts_with_all = &["lsp", "dap"],
     )]
-    repeat: usize,
+    json: bool,
 
     #[structopt(
         long = "extension",
@@ -92,11 +93,16 @@ pub struct Args {
         long = "expression",
         short = "e",
         name = "EXPRESSION",
-        help = "Expressions to evaluate."
+        help = "Expressions to evaluate.",
+        conflicts_with_all = &["lsp", "dap"],
     )]
     evaluate: Vec<String>,
 
-    #[structopt(name = "FILE", help = "Files to evaluate.")]
+    #[structopt(
+        name = "FILE",
+        help = "Files to evaluate.",
+        conflicts_with_all = &["lsp", "dap"],
+    )]
     files: Vec<PathBuf>,
 }
 
@@ -172,7 +178,7 @@ fn drain(xs: impl Iterator<Item = Message>, json: bool, stats: &mut Stats) {
 }
 
 fn interactive(ctx: &Context) -> anyhow::Result<()> {
-    let mut rl = ReadLine::new();
+    let mut rl = ReadLine::new("STARLARK_RUST_HISTFILE");
     loop {
         match rl.read_line("$> ")? {
             Some(line) => {
@@ -196,24 +202,25 @@ fn main() -> anyhow::Result<()> {
         .map_or("bzl", |x| x.as_str())
         .trim_start_match('.');
     let mut ctx = Context::new(
-        args.check,
-        args.info,
-        !args.check && !args.info,
+        if args.check {
+            ContextMode::Check
+        } else {
+            ContextMode::Run
+        },
+        !args.evaluate.is_empty() || args.interactive,
         &expand_dirs(ext, args.prelude).collect::<Vec<_>>(),
         args.interactive,
     )?;
 
     let mut stats = Stats::default();
-    for _ in 0..args.repeat {
-        for e in args.evaluate.clone() {
-            stats.increment_file();
-            drain(ctx.expression(e), args.json, &mut stats);
-        }
+    for e in args.evaluate.clone() {
+        stats.increment_file();
+        drain(ctx.expression(e), args.json, &mut stats);
+    }
 
-        for file in expand_dirs(ext, args.files.clone()) {
-            stats.increment_file();
-            drain(ctx.file(&file), args.json, &mut stats);
-        }
+    for file in expand_dirs(ext, args.files.clone()) {
+        stats.increment_file();
+        drain(ctx.file(&file), args.json, &mut stats);
     }
 
     if args.interactive {
@@ -221,9 +228,7 @@ fn main() -> anyhow::Result<()> {
     }
 
     if args.lsp {
-        ctx.check = true;
-        ctx.info = false;
-        ctx.run = false;
+        ctx.mode = ContextMode::Check;
         lsp::server(ctx)?;
     } else if args.dap {
         dap::server()
