@@ -27,7 +27,7 @@ use starlark::{
 /// A standardised set of severities.
 #[derive(Debug, Serialize, Dupe, Clone, Copy)]
 #[serde(rename_all = "lowercase")]
-pub enum Severity {
+pub(crate) enum Severity {
     Error,
     Warning,
     // Not all severities are used right now
@@ -48,15 +48,15 @@ impl Display for Severity {
 }
 
 #[derive(Debug, Clone)]
-pub struct Message {
-    pub path: String,
-    pub span: Option<ResolvedSpan>,
-    pub severity: Severity,
-    pub name: String,
-    pub description: String,
-    pub full_error_with_span: Option<String>,
+pub(crate) struct Message {
+    pub(crate) path: String,
+    pub(crate) span: Option<ResolvedSpan>,
+    pub(crate) severity: Severity,
+    pub(crate) name: String,
+    pub(crate) description: String,
+    pub(crate) full_error_with_span: Option<String>,
     /// The text referred to by span
-    pub original: Option<String>,
+    pub(crate) original: Option<String>,
 }
 
 impl Display for Message {
@@ -70,7 +70,7 @@ impl Display for Message {
 }
 
 impl Message {
-    pub fn from_anyhow(file: &str, x: anyhow::Error) -> Self {
+    pub(crate) fn from_anyhow(file: &str, x: anyhow::Error) -> Self {
         match x.downcast_ref::<Diagnostic>() {
             Some(
                 d @ Diagnostic {
@@ -82,7 +82,7 @@ impl Message {
                 let original = span.source_span().to_owned();
                 let resolved_span = span.resolve_span();
                 Self {
-                    path: span.file().filename().to_owned(),
+                    path: span.filename().to_owned(),
                     span: Some(resolved_span),
                     severity: Severity::Error,
                     name: "error".to_owned(),
@@ -103,9 +103,9 @@ impl Message {
         }
     }
 
-    pub fn from_lint(x: Lint) -> Self {
+    pub(crate) fn from_lint(x: Lint) -> Self {
         Self {
-            path: x.location.file().filename().to_owned(),
+            path: x.location.filename().to_owned(),
             span: Some(x.location.resolve_span()),
             severity: if x.serious {
                 Severity::Warning
@@ -123,8 +123,10 @@ impl Message {
 
 /// A JSON-deriving type that gives a stable interface to downstream types.
 /// Do NOT change this type, change Message instead.
+///
+/// [Linter JSON format](https://www.internalfb.com/intern/wiki/Linting/adding-linters/).
 #[derive(Debug, Clone, Serialize)]
-pub struct LintMessage {
+pub(crate) struct LintMessage {
     path: String,
     line: Option<usize>,
     char: Option<usize>,
@@ -137,7 +139,7 @@ pub struct LintMessage {
 }
 
 impl LintMessage {
-    pub fn new(x: Message) -> Self {
+    pub(crate) fn new(x: Message) -> Self {
         Self {
             path: x.path,
             line: x.span.map(|x| x.begin_line + 1),
