@@ -18,6 +18,7 @@
 use std::collections::HashMap;
 
 use gazebo::variants::VariantName;
+use num_bigint::BigInt;
 use thiserror::Error;
 
 use crate::{
@@ -25,6 +26,7 @@ use crate::{
     codemap::{CodeMap, FileSpan, Span},
     syntax::{
         ast::{AstExpr, AstLiteral, Expr},
+        lexer::TokenInt,
         AstModule,
     },
     values::num::Num,
@@ -49,6 +51,7 @@ fn duplicate_dictionary_key(module: &AstModule, res: &mut Vec<LintT<Dubious>>) {
     #[derive(PartialEq, Eq, Hash)]
     enum Key<'a> {
         Int(i32),
+        BigInt(&'a BigInt),
         Float(u64),
         String(&'a str),
         Identifier(&'a str),
@@ -57,7 +60,10 @@ fn duplicate_dictionary_key(module: &AstModule, res: &mut Vec<LintT<Dubious>>) {
     fn to_key<'a>(x: &'a AstExpr) -> Option<(Key<'a>, Span)> {
         match &**x {
             Expr::Literal(x) => match &*x {
-                AstLiteral::Int(x) => Some((Key::Int(x.node), x.span)),
+                AstLiteral::Int(x) => match &x.node {
+                    TokenInt::I32(i) => Some((Key::Int(*i), x.span)),
+                    TokenInt::BigInt(i) => Some((Key::BigInt(i), x.span)),
+                },
                 AstLiteral::Float(x) => {
                     let n = Num::from(x.node);
                     if let Some(i) = n.as_int() {
