@@ -37,7 +37,6 @@ use indexmap::Equivalent;
 
 use crate::collections::{
     hash::{BorrowHashed, Hashed},
-    idhasher::mix_u32,
     vec_map::{Bucket, VMIntoIter, VMIter, VMIterMut, VecMap},
     StarlarkHasher,
 };
@@ -46,10 +45,13 @@ use crate::collections::{
 // TODO: benchmark, is this the right threshold
 const NO_INDEX_THRESHOLD: usize = 12;
 
+/// Iterator over a small map entry references.
 pub type MHIter<'a, K, V> = VMIter<'a, K, V>;
 
+/// Iterator over a small map mutable entry references.
 pub type MHIterMut<'a, K, V> = VMIterMut<'a, K, V>;
 
+/// Iterator over a small map entries.
 pub type MHIntoIter<K, V> = VMIntoIter<K, V>;
 
 /// An memory-efficient key-value map with determinstic order.
@@ -82,6 +84,8 @@ impl<K: Debug, V: Debug> Debug for SmallMap<K, V> {
 }
 
 impl<K, V> SmallMap<K, V> {
+    /// Empty map.
+    #[inline]
     pub const fn new() -> Self {
         Self {
             entries: VecMap::new(),
@@ -89,6 +93,8 @@ impl<K, V> SmallMap<K, V> {
         }
     }
 
+    /// Create an empty map with specified capacity.
+    #[inline]
     pub fn with_capacity(n: usize) -> Self {
         if n <= NO_INDEX_THRESHOLD {
             SmallMap {
@@ -104,6 +110,7 @@ impl<K, V> SmallMap<K, V> {
     }
 
     /// Create with largest capacity which is represented by `Vec`.
+    #[inline]
     pub(crate) fn with_capacity_largest_vec() -> Self {
         Self::with_capacity(NO_INDEX_THRESHOLD)
     }
@@ -120,10 +127,12 @@ impl<K, V> SmallMap<K, V> {
         }
     }
 
+    #[inline]
     pub(crate) fn into_raw_parts(self) -> (VecMap<K, V>, Option<Box<RawTable<usize>>>) {
         (self.entries, self.index)
     }
 
+    #[inline]
     pub(crate) unsafe fn from_raw_parts(
         entries: VecMap<K, V>,
         index: Option<Box<RawTable<usize>>>,
@@ -138,38 +147,56 @@ impl<K, V> SmallMap<K, V> {
         SmallMap { entries, index }
     }
 
+    /// Key references iterator.
+    #[inline]
     pub fn keys(&self) -> impl ExactSizeIterator<Item = &K> + Clone {
         self.entries.keys()
     }
 
+    /// Value references iterator.
+    #[inline]
     pub fn values(&self) -> impl ExactSizeIterator<Item = &V> + Clone {
         self.entries.values()
     }
 
+    /// Mutable value references iterator.
+    #[inline]
     pub fn values_mut(&mut self) -> impl ExactSizeIterator<Item = &mut V> {
         self.entries.values_mut()
     }
 
+    /// Entry references iterator.
+    #[inline]
     pub fn iter(&self) -> MHIter<'_, K, V> {
         self.entries.iter()
     }
 
+    /// Entry references with hashes iterator.
+    #[inline]
     pub fn iter_hashed(&self) -> impl ExactSizeIterator<Item = (BorrowHashed<K>, &V)> {
         self.entries.iter_hashed()
     }
 
+    /// Entries with hashes iterator.
+    #[inline]
     pub fn into_iter_hashed(self) -> impl ExactSizeIterator<Item = (Hashed<K>, V)> {
         self.entries.into_iter_hashed()
     }
 
+    /// Mutable entry references iterator.
+    #[inline]
     pub fn iter_mut(&mut self) -> MHIterMut<'_, K, V> {
         self.entries.iter_mut()
     }
 
+    /// Entries iterator.
+    #[inline]
     pub fn into_iter(self) -> MHIntoIter<K, V> {
         self.entries.into_iter()
     }
 
+    /// Query the map by a prehashed key.
+    #[inline]
     pub fn get_hashed<Q>(&self, key: BorrowHashed<Q>) -> Option<&V>
     where
         Q: Equivalent<K> + ?Sized,
@@ -179,6 +206,8 @@ impl<K, V> SmallMap<K, V> {
             .map(|index| unsafe { &self.entries.get_unchecked(index).value })
     }
 
+    /// Query the map by a given key.
+    #[inline]
     pub fn get<Q>(&self, key: &Q) -> Option<&V>
     where
         Q: Hash + Equivalent<K> + ?Sized,
@@ -187,6 +216,9 @@ impl<K, V> SmallMap<K, V> {
         self.get_hashed(BorrowHashed::new(key))
     }
 
+    /// Query the map by a given key, return an index of the entry
+    /// along with the entry key and value.
+    #[inline]
     pub fn get_full<Q>(&self, key: &Q) -> Option<(usize, &K, &V)>
     where
         Q: Hash + Equivalent<K> + ?Sized,
@@ -199,6 +231,8 @@ impl<K, V> SmallMap<K, V> {
             })
     }
 
+    /// Find the index of the given hashed key.
+    #[inline]
     pub fn get_index_of_hashed<Q>(&self, key: BorrowHashed<Q>) -> Option<usize>
     where
         Q: Equivalent<K> + ?Sized,
@@ -207,17 +241,21 @@ impl<K, V> SmallMap<K, V> {
         match &self.index {
             None => self.entries.get_index_of_hashed(key),
             Some(index) => index
-                .get(mix_u32(key.hash().get()), |&index| unsafe {
+                .get(key.hash().promote(), |&index| unsafe {
                     key.key().equivalent(&self.entries.get_unchecked(index).key)
                 })
                 .copied(),
         }
     }
 
+    /// Find an entry by an index.
+    #[inline]
     pub fn get_index(&self, index: usize) -> Option<(&K, &V)> {
         self.entries.get_index(index)
     }
 
+    /// The an entry index by a given key.
+    #[inline]
     pub fn get_index_of<Q>(&self, key: &Q) -> Option<usize>
     where
         Q: Hash + Equivalent<K> + ?Sized,
@@ -226,6 +264,8 @@ impl<K, V> SmallMap<K, V> {
         self.get_index_of_hashed(BorrowHashed::new(key))
     }
 
+    /// Find a mutable value by a hashed key.
+    #[inline]
     pub fn get_mut_hashed<Q>(&mut self, key: BorrowHashed<Q>) -> Option<&mut V>
     where
         Q: Equivalent<K> + ?Sized,
@@ -236,6 +276,8 @@ impl<K, V> SmallMap<K, V> {
         Some(unsafe { &mut self.entries.buckets.get_unchecked_mut(i).value })
     }
 
+    /// Find the entry by a given key.
+    #[inline]
     pub fn get_mut<Q>(&mut self, key: &Q) -> Option<&mut V>
     where
         Q: Hash + Equivalent<K> + ?Sized,
@@ -244,6 +286,8 @@ impl<K, V> SmallMap<K, V> {
         self.get_mut_hashed(BorrowHashed::new(key))
     }
 
+    /// Find if an entry by a given prehashed key exists.
+    #[inline]
     pub fn contains_key_hashed<Q>(&self, key: BorrowHashed<Q>) -> bool
     where
         Q: Equivalent<K> + ?Sized,
@@ -252,6 +296,8 @@ impl<K, V> SmallMap<K, V> {
         self.get_index_of_hashed(key).is_some()
     }
 
+    /// Find if an entry by a given key exists.
+    #[inline]
     pub fn contains_key<Q>(&self, key: &Q) -> bool
     where
         Q: Hash + Equivalent<K> + ?Sized,
@@ -260,6 +306,8 @@ impl<K, V> SmallMap<K, V> {
         self.contains_key_hashed(BorrowHashed::new(key))
     }
 
+    /// Reserve capacity for at least `additional` more elements to be inserted.
+    #[inline]
     pub fn reserve(&mut self, additional: usize)
     where
         K: Eq,
@@ -272,6 +320,8 @@ impl<K, V> SmallMap<K, V> {
         }
     }
 
+    /// Current map capacity.
+    #[inline]
     pub fn capacity(&self) -> usize {
         self.entries.capacity()
     }
@@ -300,29 +350,28 @@ impl<K, V> SmallMap<K, V> {
         debug_assert!(capacity >= self.entries.len());
         let mut index = RawTable::with_capacity(capacity);
         for (i, b) in self.entries.buckets.iter().enumerate() {
-            index.insert_no_grow(mix_u32(b.hash.get()), i);
+            index.insert_no_grow(b.hash.promote(), i);
         }
         self.index = Some(box index);
     }
 
     /// Hasher for index resize.
+    #[inline(always)]
     fn hasher<'a>(entries: &'a VecMap<K, V>) -> impl Fn(&usize) -> u64 + 'a {
         move |&index| {
             debug_assert!(index < entries.len());
-            unsafe { mix_u32(entries.buckets.get_unchecked(index).hash.get()) }
+            unsafe { entries.buckets.get_unchecked(index).hash.promote() }
         }
     }
 
+    /// Insert an entry into the map without checking for a duplicate key.
+    #[inline]
     fn insert_unique_unchecked(&mut self, key: Hashed<K>, val: V) {
         let hash = key.hash();
         let entry_index = self.entries.len();
         self.entries.insert_unique_unchecked(key, val);
         if let Some(index) = &mut self.index {
-            index.insert(
-                mix_u32(hash.get()),
-                entry_index,
-                Self::hasher(&self.entries),
-            );
+            index.insert(hash.promote(), entry_index, Self::hasher(&self.entries));
         } else if self.entries.len() == NO_INDEX_THRESHOLD + 1 {
             self.create_index(self.entries.len());
         } else {
@@ -330,6 +379,8 @@ impl<K, V> SmallMap<K, V> {
         }
     }
 
+    /// Insert a key-value pair into the map.
+    #[inline]
     pub fn insert_hashed(&mut self, key: Hashed<K>, val: V) -> Option<V>
     where
         K: Eq,
@@ -349,6 +400,8 @@ impl<K, V> SmallMap<K, V> {
         }
     }
 
+    /// Insert a key-value pair into the map.
+    #[inline]
     pub fn insert(&mut self, key: K, val: V) -> Option<V>
     where
         K: Hash + Eq,
@@ -356,6 +409,9 @@ impl<K, V> SmallMap<K, V> {
         self.insert_hashed(Hashed::new(key), val)
     }
 
+    /// Remove the entry for the key.
+    ///
+    /// This operation is linear in the number of entries in the map.
     pub fn remove_hashed<Q>(&mut self, key: BorrowHashed<Q>) -> Option<V>
     where
         Q: ?Sized + Equivalent<K>,
@@ -364,6 +420,9 @@ impl<K, V> SmallMap<K, V> {
         self.remove_hashed_entry(key).map(|(_k, v)| v)
     }
 
+    /// Remove the entry for the key.
+    ///
+    /// This operation is linear in the number of entries in the map.
     pub fn remove_hashed_entry<Q>(&mut self, key: BorrowHashed<Q>) -> Option<(K, V)>
     where
         Q: ?Sized + Equivalent<K>,
@@ -372,7 +431,7 @@ impl<K, V> SmallMap<K, V> {
         let hash = key.hash();
         if let Some(index) = &mut self.index {
             let entries = &self.entries;
-            let i = index.remove_entry(mix_u32(hash.get()), |&i| unsafe {
+            let i = index.remove_entry(hash.promote(), |&i| unsafe {
                 key.key().equivalent(&entries.get_unchecked(i).key)
             })?;
             unsafe {
@@ -394,6 +453,7 @@ impl<K, V> SmallMap<K, V> {
         }
     }
 
+    /// Remove the entry for the key.
     pub fn remove<Q>(&mut self, key: &Q) -> Option<V>
     where
         Q: ?Sized + Hash + Equivalent<K>,
@@ -402,6 +462,7 @@ impl<K, V> SmallMap<K, V> {
         self.remove_hashed(BorrowHashed::new(key))
     }
 
+    /// Remove the entry for the key.
     pub fn remove_entry<Q>(&mut self, key: &Q) -> Option<(K, V)>
     where
         Q: ?Sized + Hash + Equivalent<K>,
@@ -410,6 +471,8 @@ impl<K, V> SmallMap<K, V> {
         self.remove_hashed_entry(BorrowHashed::new(key))
     }
 
+    /// Get the entry (occupied or not) for the key.
+    #[inline]
     pub fn entry_hashed(&mut self, key: Hashed<K>) -> Entry<'_, K, V>
     where
         K: Eq,
@@ -435,8 +498,7 @@ impl<K, V> SmallMap<K, V> {
             None => None,
             Some(Bucket { key, value, hash }) => {
                 if let Some(index) = &mut self.index {
-                    let removed =
-                        index.remove_entry(mix_u32(hash.get()), |&i| i == self.entries.len());
+                    let removed = index.remove_entry(hash.promote(), |&i| i == self.entries.len());
                     debug_assert!(removed.unwrap() == self.entries.len());
                 }
                 Some((key, value))
@@ -444,6 +506,8 @@ impl<K, V> SmallMap<K, V> {
         }
     }
 
+    /// Get the entry (occupied or not) for the key.
+    #[inline]
     pub fn entry(&mut self, key: K) -> Entry<'_, K, V>
     where
         K: Eq + Hash,
@@ -451,14 +515,22 @@ impl<K, V> SmallMap<K, V> {
         self.entry_hashed(Hashed::new(key))
     }
 
+    /// Is the map empty?
+    #[inline]
     pub fn is_empty(&self) -> bool {
         self.entries.is_empty()
     }
 
+    /// Get the number of elements in the map.
+    #[inline]
     pub fn len(&self) -> usize {
         self.entries.len()
     }
 
+    /// Remove all elements from the map.
+    ///
+    /// Retain the capacity.
+    #[inline]
     pub fn clear(&mut self) {
         self.entries.clear();
         if let Some(index) = &mut self.index {
@@ -489,30 +561,43 @@ impl<K, V> SmallMap<K, V> {
     }
 }
 
+/// Reference to the actual entry in the map.
 pub struct OccupiedEntry<'a, K, V> {
     key: &'a K,
     value: &'a mut V,
 }
 
+/// Reference to a vacant entry in the map.
+///
+/// This can be used to insert an entry into the map.
 pub struct VacantEntry<'a, K, V> {
     key: Hashed<K>,
     map: &'a mut SmallMap<K, V>,
 }
 
+/// Occupied or vacant entry.
 pub enum Entry<'a, K, V> {
+    /// Occupied entry.
     Occupied(OccupiedEntry<'a, K, V>),
+    /// No entry for given key.
     Vacant(VacantEntry<'a, K, V>),
 }
 
 impl<'a, K, V> OccupiedEntry<'a, K, V> {
+    /// Key for this entry.
+    #[inline]
     pub fn key(&self) -> &K {
         self.key
     }
 
+    /// Value for this entry.
+    #[inline]
     pub fn get(&self) -> &V {
         self.value
     }
 
+    /// Mutable reference to the value in the entry.
+    #[inline]
     pub fn get_mut(&mut self) -> &mut V {
         self.value
     }
@@ -522,11 +607,15 @@ impl<'a, K, V> VacantEntry<'a, K, V>
 where
     K: Eq,
 {
+    /// Key for this entry.
+    #[inline]
     pub fn key(&self) -> &K {
         self.key.key()
     }
 
+    /// Insert the value into the entry.
     // NOTE(nga): `VacantEntry::insert` is supposed to return `&'a mut V`
+    #[inline]
     pub fn insert(self, value: V) {
         self.map.insert_unique_unchecked(self.key, value);
     }
@@ -536,6 +625,8 @@ impl<'a, K, V> Entry<'a, K, V>
 where
     K: Eq,
 {
+    /// Key for this entry.
+    #[inline]
     pub fn key(&self) -> &K {
         match self {
             Entry::Occupied(e) => e.key(),
@@ -576,6 +667,7 @@ impl<K, V> IntoIterator for SmallMap<K, V> {
     type Item = (K, V);
     type IntoIter = MHIntoIter<K, V>;
 
+    #[inline]
     fn into_iter(self) -> Self::IntoIter {
         self.into_iter()
     }
@@ -585,6 +677,7 @@ impl<'a, K, V> IntoIterator for &'a SmallMap<K, V> {
     type Item = (&'a K, &'a V);
     type IntoIter = MHIter<'a, K, V>;
 
+    #[inline]
     fn into_iter(self) -> Self::IntoIter {
         self.iter()
     }
@@ -594,6 +687,7 @@ impl<'a, K, V> IntoIterator for &'a mut SmallMap<K, V> {
     type Item = (&'a K, &'a mut V);
     type IntoIter = MHIterMut<'a, K, V>;
 
+    #[inline]
     fn into_iter(self) -> Self::IntoIter {
         self.iter_mut()
     }
