@@ -17,7 +17,13 @@
 
 use crate::{
     collections::SmallMap,
-    eval::{Def, Evaluator, FrozenDef, ScopeNames},
+    eval::{
+        compiler::{
+            def::{Def, FrozenDef},
+            scope::ScopeNames,
+        },
+        Evaluator,
+    },
     values::{Value, ValueLike},
 };
 
@@ -73,10 +79,10 @@ mod tests {
     #[starlark_module]
     fn debugger(builder: &mut GlobalsBuilder) {
         fn debug_inspect_stack() -> anyhow::Result<Vec<String>> {
-            Ok(eval.call_stack().map(ToString::to_string))
+            Ok(eval.call_stack().into_frames().map(ToString::to_string))
         }
 
-        fn debug_inspect_variables() -> anyhow::Result<Dict<'v>> {
+        fn debug_inspect_variables<'v>() -> anyhow::Result<Dict<'v>> {
             let mut sm = SmallMap::new();
             for (k, v) in eval.local_variables() {
                 sm.insert_hashed(heap.alloc_str(&k).get_hashed(), v);
@@ -97,7 +103,7 @@ def assert_stack(want):
 
 assert_stack([])
 
-def f(): assert_stack(["assert.bzl.g", "assert.bzl.f"])
+def f(): assert_stack(["g", "f"])
 def g(): f()
 g()
 "#,
