@@ -28,7 +28,11 @@ use annotate_snippets::{
 };
 
 pub use crate::analysis::Lint;
-use crate::codemap::{CodeMap, FileSpan, Span};
+use crate::{
+    codemap::{CodeMap, FileSpan, Span},
+    eval::CallStack,
+    values::string::{fast_string, CharIndex},
+};
 
 pub(crate) mod did_you_mean;
 
@@ -46,11 +50,11 @@ pub struct Diagnostic {
     pub span: Option<FileSpan>,
 
     /// Call stack of what called what. Most recent frames are at the end.
-    pub call_stack: Vec<Frame>,
+    pub call_stack: CallStack,
 }
 
 /// A frame of the call-stack.
-#[derive(Debug, PartialEq, Eq, Hash)]
+#[derive(Debug, PartialEq, Eq, Hash, Clone)]
 pub struct Frame {
     /// The name of the entry on the call-stack.
     pub name: String,
@@ -68,6 +72,48 @@ impl Display for Frame {
     }
 }
 
+fn truncate_snippet(snippet: &str, max_len: usize) -> (&str, &str) {
+    let ddd = "...";
+    assert!(max_len >= ddd.len());
+    match fast_string::split_at(snippet, CharIndex(max_len - ddd.len())) {
+        None => (snippet, ""),
+        Some((_, b)) if b.chars().nth(3).is_none() => (snippet, ""),
+        Some((a, _)) => (a, "..."),
+    }
+}
+
+impl Frame {
+    pub(crate) fn write_two_lines(
+        &self,
+        indent: &str,
+        caller: &str,
+        write: &mut dyn fmt::Write,
+    ) -> fmt::Result {
+        if let Some(location) = &self.location {
+            let line = location
+                .file
+                .source_line_at_pos(location.span.begin())
+                .trim();
+            let (line, ddd) = truncate_snippet(line, 50);
+            writeln!(
+                write,
+                "{}* {}:{}, in {}",
+                indent,
+                location.file.filename(),
+                location.file.find_line(location.span.begin()) + 1,
+                // Note we print caller function here as in Python, not callee,
+                // so in the stack trace, top frame is printed without executed function name.
+                caller,
+            )?;
+            writeln!(write, "{}  {}{}", indent, line, ddd)?;
+        } else {
+            // Python just omits builtin functions in the traceback.
+            writeln!(write, "{}File <builtin>, in {}", indent, caller)?;
+        }
+        Ok(())
+    }
+}
+
 impl Error for Diagnostic {
     fn source(&self) -> Option<&(dyn Error + 'static)> {
         // We do have an underlying source (namely `self.message`), but if we return
@@ -107,7 +153,7 @@ impl Diagnostic {
                 let mut err = Self {
                     message: err,
                     span: None,
-                    call_stack: Vec::new(),
+                    call_stack: CallStack::default(),
                 };
                 f(&mut err);
                 err.into()
@@ -124,7 +170,7 @@ impl Diagnostic {
     }
 
     /// Set the [`Diagnostic::call_stack`] field, unless it's already been set.
-    pub fn set_call_stack(&mut self, call_stack: impl FnOnce() -> Vec<Frame>) {
+    pub fn set_call_stack(&mut self, call_stack: impl FnOnce() -> CallStack) {
         if self.call_stack.is_empty() {
             // We want the best call stack, which is likely the first person to set it
             self.call_stack = call_stack();
@@ -151,17 +197,6 @@ impl Display for Diagnostic {
     }
 }
 
-struct CallStackFmt<'a>(&'a Vec<Frame>);
-
-impl Display for CallStackFmt<'_> {
-    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
-        for x in self.0.iter() {
-            writeln!(f, "* {}", x)?;
-        }
-        Ok(())
-    }
-}
-
 /////////////////////////////////////////////////////////////////////
 // DISPLAY RELATED UTILITIES
 // Since formatting these types is difficult, we reuse the Rust compiler
@@ -224,7 +259,7 @@ fn get_display_list_for_diagnostic<'a>(
 }
 
 fn diagnostic_display(diagnostic: &Diagnostic, f: &mut Formatter<'_>) -> fmt::Result {
-    CallStackFmt(&diagnostic.call_stack).fmt(f)?;
+    write!(f, "{}", &diagnostic.call_stack)?;
     let annotation_label = format!("{:#}", diagnostic.message);
     // I set color to false here to make the comparison easier with tests (coloring
     // adds in pretty strange unicode chars).
@@ -233,8 +268,29 @@ fn diagnostic_display(diagnostic: &Diagnostic, f: &mut Formatter<'_>) -> fmt::Re
 }
 
 fn diagnostic_stderr(diagnostic: &Diagnostic) {
-    eprint!("{}", CallStackFmt(&diagnostic.call_stack));
+    eprint!("{}", diagnostic.call_stack);
     let annotation_label = format!("{:#}", diagnostic.message);
     let display_list = get_display_list_for_diagnostic(&annotation_label, diagnostic, true);
     eprintln!("{}", display_list);
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::errors::truncate_snippet;
+
+    #[test]
+    fn test_truncate_snippet() {
+        assert_eq!(("", ""), truncate_snippet("", 5));
+        assert_eq!(("a", ""), truncate_snippet("a", 5));
+        assert_eq!(("ab", ""), truncate_snippet("ab", 5));
+        assert_eq!(("abc", ""), truncate_snippet("abc", 5));
+        assert_eq!(("abcd", ""), truncate_snippet("abcd", 5));
+        assert_eq!(("abcde", ""), truncate_snippet("abcde", 5));
+        assert_eq!(("ab", "..."), truncate_snippet("abcdef", 5));
+        assert_eq!(("ab", "..."), truncate_snippet("abcdefg", 5));
+        assert_eq!(("ab", "..."), truncate_snippet("abcdefgh", 5));
+        assert_eq!(("ab", "..."), truncate_snippet("abcdefghi", 5));
+        assert_eq!(("Київ", ""), truncate_snippet("Київ", 5));
+        assert_eq!(("па", "..."), truncate_snippet("паляниця", 5));
+    }
+}
