@@ -29,8 +29,7 @@ use crate::{
             },
             writer::BcWriter,
         },
-        compiler::scope::Captured,
-        fragment::{expr::ExprCompiled, span::IrSpanned, stmt::AssignModifyLhs},
+        compiler::{expr::ExprCompiled, scope::Captured, span::IrSpanned, stmt::AssignModifyLhs},
         runtime::call_stack::FrozenFileSpan,
     },
     syntax::ast::AssignOp,
