@@ -35,16 +35,16 @@ use crate::{
             },
             writer::BcWriter,
         },
-        fragment::{
+        compiler::{
             call::{ArgsCompiledValue, CallCompiled},
+            def::FrozenDef,
             span::IrSpanned,
         },
         runtime::call_stack::FrozenFileSpan,
-        FrozenDef,
     },
     values::{
-        function::NativeFunction, known_methods::get_known_method, typed::FrozenValueTyped,
-        FrozenValue,
+        function::NativeFunction, types::known_methods::get_known_method, FrozenValue,
+        FrozenValueTyped,
     },
 };
 
