@@ -39,8 +39,7 @@ use crate::{
         runtime::{call_stack::FrozenFileSpan, slots::LocalSlotId},
     },
     values::{
-        known_methods::KnownMethod, typed::FrozenValueTyped, FrozenRef, FrozenStringValue,
-        FrozenValue, StarlarkValue,
+        types::known_methods::KnownMethod, FrozenRef, FrozenValue, FrozenValueTyped, StarlarkValue,
     },
 };
 
@@ -307,22 +306,6 @@ impl BcInstrArg for Option<FrozenValue> {
     }
 }
 
-impl BcInstrArg for FrozenStringValue {
-    fn fmt_append(param: &Self, _ip: BcAddr, f: &mut dyn Write) -> fmt::Result {
-        write!(f, " {}", TruncateValueRepr(param.unpack()))
-    }
-
-    fn visit_jump_addr(_param: &Self, _consumer: &mut dyn FnMut(BcAddrOffset)) {}
-
-    fn pops_stack(_: &Self) -> u32 {
-        0
-    }
-
-    fn pushes_stack(_: &Self) -> u32 {
-        0
-    }
-}
-
 impl BcInstrArg for String {
     fn fmt_append(param: &Self, _ip: BcAddr, f: &mut dyn Write) -> fmt::Result {
         write!(f, "{:?}", param)
