@@ -38,25 +38,27 @@ use crate::{
             stack_ptr::BcStackPtr,
             stack_values::BcStackValues,
         },
-        compiler::{add_span_to_expr_error, expr_throw, scope::Captured, EvalException},
-        fragment::{
-            def::{DefInfo, ParameterCompiled},
+        compiler::{
+            add_span_to_expr_error,
+            def::{Def, FrozenDef, ParameterCompiled},
             expr::{get_attr_hashed_bind, get_attr_hashed_raw, EvalError, MemberOrValue},
+            expr_throw,
+            scope::Captured,
             span::IrSpanned,
             stmt::{add_assign, before_stmt, bit_or_assign, possible_gc, AssignError},
+            EvalException,
         },
         runtime::{call_stack::FrozenFileSpan, slots::LocalSlotId},
-        Arguments, Def, Evaluator, FrozenDef, ParametersSpec,
+        Arguments, DefInfo, Evaluator, ParametersSpec,
     },
     values::{
         dict::Dict,
         function::NativeFunction,
-        known_methods::KnownMethod,
         list::List,
         string::interpolation::{format_one, percent_s_one},
-        typed::FrozenValueTyped,
+        types::known_methods::KnownMethod,
         typing::TypeCompiled,
-        FrozenRef, FrozenStringValue, FrozenValue, Heap, StarlarkValue, StringValue, Value,
+        FrozenRef, FrozenStringValue, FrozenValue, FrozenValueTyped, Heap, StarlarkValue, Value,
     },
 };
 
@@ -649,8 +651,8 @@ impl InstrUnOpImpl for InstrMinusImpl {
 
 impl InstrUnOpImpl for InstrBitNotImpl {
     #[inline(always)]
-    fn eval<'v>(v: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
-        Ok(Value::new_int(!v.to_int()?))
+    fn eval<'v>(v: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        v.bit_not(heap)
     }
 }
 
@@ -736,19 +738,6 @@ pub(crate) type InstrNotIn = InstrBinOp<InstrNotInImpl>;
 impl InstrBinOpImpl for InstrAddImpl {
     #[inline(always)]
     fn eval<'v>(l: Value<'v>, r: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
-        // Addition of string is super common and pretty cheap, so have a special case for it.
-        if let Some(ls) = l.unpack_str() {
-            if let Some(rs) = r.unpack_str() {
-                if ls.is_empty() {
-                    return Ok(r);
-                } else if rs.is_empty() {
-                    return Ok(l);
-                } else {
-                    return Ok(heap.alloc_str_concat(ls, rs).to_value());
-                }
-            }
-        }
-
         l.add(r, heap)
     }
 }
@@ -797,8 +786,8 @@ impl InstrBinOpImpl for InstrDivideImpl {
 
 impl InstrBinOpImpl for InstrBitAndImpl {
     #[inline(always)]
-    fn eval<'v>(v0: Value<'v>, v1: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
-        v0.bit_and(v1)
+    fn eval<'v>(v0: Value<'v>, v1: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        v0.bit_and(v1, heap)
     }
 }
 
@@ -818,22 +807,22 @@ impl InstrBinOpImpl for InstrBitOrAssignImpl {
 
 impl InstrBinOpImpl for InstrBitXorImpl {
     #[inline(always)]
-    fn eval<'v>(v0: Value<'v>, v1: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
-        v0.bit_xor(v1)
+    fn eval<'v>(v0: Value<'v>, v1: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        v0.bit_xor(v1, heap)
     }
 }
 
 impl InstrBinOpImpl for InstrLeftShiftImpl {
     #[inline(always)]
-    fn eval<'v>(v0: Value<'v>, v1: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
-        v0.left_shift(v1)
+    fn eval<'v>(v0: Value<'v>, v1: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        v0.left_shift(v1, heap)
     }
 }
 
 impl InstrBinOpImpl for InstrRightShiftImpl {
     #[inline(always)]
-    fn eval<'v>(v0: Value<'v>, v1: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
-        v0.right_shift(v1)
+    fn eval<'v>(v0: Value<'v>, v1: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        v0.right_shift(v1, heap)
     }
 }
 
@@ -869,7 +858,7 @@ impl InstrNoFlowImpl for InstrPercentSOneImpl {
         (before, after): &Self::Arg,
         arg: Value<'v>,
     ) -> anyhow::Result<Value<'v>> {
-        percent_s_one(before.as_str(), arg, after.as_str(), eval.heap()).map(StringValue::to_value)
+        percent_s_one(before.as_str(), arg, after.as_str(), eval.heap()).map(|v| v.to_value())
     }
 }
 
@@ -947,7 +936,7 @@ pub(crate) type InstrType = InstrUnOp<InstrTypeImpl>;
 impl InstrUnOpImpl for InstrTypeImpl {
     #[inline(always)]
     fn eval<'v>(v: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
-        Ok(v.get_type_value().unpack().to_value())
+        Ok(v.get_type_value().to_frozen_value().to_value())
     }
 }
 
@@ -1430,14 +1419,13 @@ impl InstrNoFlowImpl for InstrDefImpl {
         // index in the parameter mapping, and it messes up the indexes
         let mut i = 0;
         for x in &def_data.params {
-            if let Some(t) = x.ty() {
+            if let Some((name, Some(t))) = x.name_ty() {
                 assert!(*t == pop_index);
                 let v = pop[pop_index as usize];
                 pop_index += 1;
-                let name = x.name().unwrap_or("unknown").to_owned();
                 parameter_types.push((
                     i,
-                    name,
+                    name.name.clone(),
                     v,
                     expr_throw(TypeCompiled::new(v, eval.heap()), x.span, eval).map_err(|e| e.0)?,
                 ));
