@@ -19,6 +19,7 @@
 
 pub(crate) mod addr;
 pub(crate) mod bytecode;
+pub(crate) mod call;
 pub(crate) mod compiler;
 pub(crate) mod frame;
 pub(crate) mod if_debug;
@@ -28,7 +29,7 @@ pub(crate) mod instr_impl;
 pub(crate) mod instrs;
 pub(crate) mod opcode;
 pub(crate) mod repr;
-pub(crate) mod spans;
+pub(crate) mod slow_arg;
 pub(crate) mod stack_ptr;
 pub(crate) mod stack_values;
 pub(crate) mod writer;
