@@ -15,7 +15,17 @@
  * limitations under the License.
  */
 
+pub(crate) mod call;
+pub(crate) mod compr;
+pub(crate) mod def;
+pub(crate) mod expr;
+pub(crate) mod expr_bool;
+pub(crate) mod known;
+pub(crate) mod module;
 pub(crate) mod scope;
+pub(crate) mod small_vec_1;
+pub(crate) mod span;
+pub(crate) mod stmt;
 
 use std::fmt::Debug;
 
@@ -27,9 +37,9 @@ use crate::{
     environment::Globals,
     errors::Diagnostic,
     eval::{
-        compiler::scope::{ScopeData, ScopeId},
+        compiler::scope::{ScopeData, ScopeId, ScopeNames},
         runtime::call_stack::FrozenFileSpan,
-        Evaluator, ScopeNames,
+        Evaluator,
     },
     values::{FrozenRef, FrozenValue},
 };
@@ -42,7 +52,7 @@ pub(crate) struct EvalException(pub(crate) anyhow::Error);
 #[inline(never)]
 fn add_span_to_error(e: anyhow::Error, span: FrozenFileSpan, eval: &Evaluator) -> anyhow::Error {
     Diagnostic::modify(e, |d: &mut Diagnostic| {
-        d.set_span(span.span, &span.file);
+        d.set_span(span.span(), &span.file());
         d.set_call_stack(|| eval.call_stack.to_diagnostic_frames());
     })
 }
