@@ -20,11 +20,10 @@
 
 use std::{intrinsics::unlikely, mem, time::Instant};
 
-pub(crate) use compiler::scope::ScopeNames;
-pub(crate) use fragment::def::{Def, FrozenDef};
 use gazebo::prelude::*;
 pub use runtime::{
     arguments::{Arguments, ParametersParser, ParametersSpec},
+    call_stack::CallStack,
     evaluator::Evaluator,
     file_loader::{FileLoader, ReturnFileLoader},
 };
@@ -32,23 +31,21 @@ pub use runtime::{
 use crate::{
     collections::symbol_map::Symbol,
     environment::Globals,
-    eval::{
-        compiler::{
-            scope::{CompilerAstMap, Scope, ScopeData},
-            Compiler, Constants,
-        },
-        fragment::def::DefInfo,
+    eval::compiler::{
+        scope::{CompilerAstMap, Scope, ScopeData},
+        Compiler, Constants,
     },
     syntax::ast::AstModule,
     values::{docs::DocString, Value},
 };
 
 pub(crate) mod bc;
-mod compiler;
-mod fragment;
+pub(crate) mod compiler;
 pub(crate) mod runtime;
 pub use runtime::profile::ProfileMode;
 
+use crate::eval::compiler::def::DefInfo;
+
 #[cfg(test)]
 mod tests;
 
@@ -58,7 +55,11 @@ impl<'v, 'a> Evaluator<'v, 'a> {
     pub fn eval_module(&mut self, ast: AstModule, globals: &Globals) -> anyhow::Result<Value<'v>> {
         let start = Instant::now();
 
-        let AstModule { codemap, statement } = ast;
+        let AstModule {
+            codemap,
+            statement,
+            dialect,
+        } = ast;
 
         let codemap = self
             .module_env
@@ -84,6 +85,7 @@ impl<'v, 'a> Evaluator<'v, 'a> {
             &mut statement,
             globals,
             codemap,
+            &dialect,
         );
 
         // We want to grab the first error only, with ownership, so drop all but the first
