@@ -19,7 +19,6 @@ use std::{cell::Cell, cmp, collections::HashMap, convert::TryInto, intrinsics::u
 
 use either::Either;
 use gazebo::{
-    cell::ARef,
     coerce::{coerce, Coerce},
     prelude::*,
 };
@@ -30,12 +29,15 @@ use crate as starlark;
 use crate::{
     collections::{
         symbol_map::{Symbol, SymbolMap},
-        BorrowHashed, Hashed, SmallMap,
+        Hashed, SmallMap,
     },
     eval::Evaluator,
     values::{
-        dict::Dict, docs, docs::DocString, Freezer, FrozenValue, Heap, StringValue, Trace, Tracer,
-        UnpackValue, Value, ValueError, ValueLike,
+        dict::{Dict, DictRef},
+        docs,
+        docs::DocString,
+        Freezer, FrozenValue, Heap, StringValue, Trace, Tracer, UnpackValue, Value, ValueError,
+        ValueLike,
     },
 };
 
@@ -58,8 +60,9 @@ pub(crate) enum FunctionError {
     ArgsArrayIsNotIterable,
     #[error("The argument provided for **kwargs is not a dictionary")]
     KwArgsIsNotDict,
-    #[error("Wrong number of positional parameters, expected between {0} and {0}, got {1}")]
-    WrongNumberOfParameters(usize, usize, usize),
+    #[error("Wrong number of positional parameters, expected {}, got {got}",
+        if min == max {min.to_string()} else {format!("between {} and {}", min, max)})]
+    WrongNumberOfParameters { min: usize, max: usize, got: usize },
 }
 
 #[derive(Debug, Clone, Coerce, PartialEq)]
@@ -327,6 +330,7 @@ impl<V> ParametersSpec<V> {
 }
 
 impl<'v, V: ValueLike<'v>> ParametersSpec<V> {
+    /// Number of function parameters.
     pub fn len(&self) -> usize {
         self.kinds.len()
     }
@@ -335,16 +339,19 @@ impl<'v, V: ValueLike<'v>> ParametersSpec<V> {
     /// using the supplied [`ParametersSpec`].
     pub fn collect(
         &self,
-        args: Arguments<'v, '_>,
+        args: &Arguments<'v, '_>,
         slots: &[Cell<Option<Value<'v>>>],
         heap: &'v Heap,
     ) -> anyhow::Result<()> {
         self.collect_inline(args, slots, heap)
     }
 
+    /// Collect `N` arguments.
+    ///
+    /// This function is called by generated code.
     pub fn collect_into<const N: usize>(
         &self,
-        args: Arguments<'v, '_>,
+        args: &Arguments<'v, '_>,
         heap: &'v Heap,
     ) -> anyhow::Result<[Cell<Option<Value<'v>>>; N]> {
         let slots = [(); N].map(|_| Cell::new(None));
@@ -357,7 +364,7 @@ impl<'v, V: ValueLike<'v>> ParametersSpec<V> {
     #[inline(always)]
     pub(crate) fn collect_inline(
         &self,
-        args: Arguments<'v, '_>,
+        args: &Arguments<'v, '_>,
         slots: &[Cell<Option<Value<'v>>>],
         heap: &'v Heap,
     ) -> anyhow::Result<()> {
@@ -382,7 +389,7 @@ impl<'v, V: ValueLike<'v>> ParametersSpec<V> {
 
     fn collect_slow(
         &self,
-        args: Arguments<'v, '_>,
+        args: &Arguments<'v, '_>,
         slots: &[Cell<Option<Value<'v>>>],
         heap: &'v Heap,
     ) -> anyhow::Result<()> {
@@ -495,9 +502,10 @@ impl<'v, V: ValueLike<'v>> ParametersSpec<V> {
                         match StringValue::new(*k.key()) {
                             None => return Err(FunctionError::ArgsValueIsNotString.into()),
                             Some(s) => {
-                                let repeat = match self.names.get_hashed_str(
-                                    BorrowHashed::new_unchecked(k.hash(), s.as_str()),
-                                ) {
+                                let repeat = match self
+                                    .names
+                                    .get_hashed_string_value(Hashed::new_unchecked(k.hash(), s))
+                                {
                                     None => kwargs.insert(Hashed::new_unchecked(k.hash(), s), v),
                                     Some(i) => {
                                         let this_slot = &slots[*i];
@@ -627,7 +635,7 @@ impl<'v, V: ValueLike<'v>> ParametersSpec<V> {
     /// Create a [`ParametersParser`] for given arguments.
     pub fn parser<R, F>(
         &self,
-        args: Arguments<'v, '_>,
+        args: &Arguments<'v, '_>,
         eval: &mut Evaluator<'v, '_>,
         k: F,
     ) -> anyhow::Result<R>
@@ -667,6 +675,7 @@ impl<'v> ParametersSpec<Value<'v>> {
 pub struct ParametersParser<'v, 'a>(std::slice::Iter<'a, Cell<Option<Value<'v>>>>);
 
 impl<'v, 'a> ParametersParser<'v, 'a> {
+    /// Create a parameter parser, which stored parameters into provided slots reference.
     pub fn new(slots: &'a [Cell<Option<Value<'v>>>]) -> Self {
         Self(slots.iter())
     }
@@ -679,11 +688,6 @@ impl<'v, 'a> ParametersParser<'v, 'a> {
         v.get()
     }
 
-    pub fn this<T: UnpackValue<'v>>(&self, this: Option<Value<'v>>) -> anyhow::Result<T> {
-        let this = this.ok_or(ValueError::MissingThis)?;
-        T::unpack_named_param(this, "this")
-    }
-
     /// Obtain the next parameter, corresponding to [`ParametersSpec::optional`].
     /// It is an error to request more parameters than were specified.
     /// The `name` is only used for error messages.
@@ -709,6 +713,8 @@ impl<'v, 'a> ParametersParser<'v, 'a> {
     }
 }
 
+/// Arguments object is passed from the starlark interpreter to function implementation
+/// when evaluation function or method calls.
 #[derive(Default, Clone, Copy, Dupe)]
 pub struct Arguments<'v, 'a> {
     /// Positional arguments.
@@ -791,7 +797,7 @@ impl<'v, 'a> Arguments<'v, 'a> {
     /// will _not_ have been validated to be strings (as they must be).
     /// The arguments may also overlap with named, which would be an error.
     #[inline(always)]
-    pub fn unpack_kwargs(&self) -> anyhow::Result<Option<ARef<'v, Dict<'v>>>> {
+    pub fn unpack_kwargs(&self) -> anyhow::Result<Option<DictRef<'v>>> {
         match self.kwargs {
             None => Ok(None),
             Some(kwargs) => match Dict::from_value(kwargs) {
@@ -874,15 +880,25 @@ impl<'v, 'a> Arguments<'v, 'a> {
                 .copied()
                 .chain(x.args.unwrap().iterate(heap)?)
                 .collect::<Vec<_>>();
-            xs.as_slice()
-                .try_into()
-                .map_err(|_| FunctionError::WrongNumberOfParameters(N, N, x.pos.len()).into())
+            xs.as_slice().try_into().map_err(|_| {
+                FunctionError::WrongNumberOfParameters {
+                    min: N,
+                    max: N,
+                    got: x.pos.len(),
+                }
+                .into()
+            })
         }
 
         if self.args.is_none() {
-            self.pos
-                .try_into()
-                .map_err(|_| FunctionError::WrongNumberOfParameters(N, N, self.pos.len()).into())
+            self.pos.try_into().map_err(|_| {
+                FunctionError::WrongNumberOfParameters {
+                    min: N,
+                    max: N,
+                    got: self.pos.len(),
+                }
+                .into()
+            })
         } else {
             rare(self, heap)
         }
@@ -917,10 +933,12 @@ impl<'v, 'a> Arguments<'v, 'a> {
                 }
                 Ok((required, optional))
             } else {
-                Err(
-                    FunctionError::WrongNumberOfParameters(REQUIRED, REQUIRED + OPTIONAL, xs.len())
-                        .into(),
-                )
+                Err(FunctionError::WrongNumberOfParameters {
+                    min: REQUIRED,
+                    max: REQUIRED + OPTIONAL,
+                    got: xs.len(),
+                }
+                .into())
             }
         }
 
@@ -995,7 +1013,7 @@ impl Arguments<'_, '_> {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::*;
 
     #[test]
@@ -1081,15 +1099,15 @@ mod test {
         p.kwargs = Some(heap.alloc(Dict::default()));
         assert!(p.no_named_args().is_ok());
         let mut sm = SmallMap::new();
-        sm.insert_hashed(heap.alloc_str_hashed("test"), Value::new_none());
-        p.kwargs = Some(heap.alloc(Dict::new(sm)));
+        sm.insert_hashed(heap.alloc_str("test").get_hashed(), Value::new_none());
+        p.kwargs = Some(heap.alloc(Dict::new(coerce(sm))));
         assert!(p.no_named_args().is_err());
 
         // Test named arguments work properly
         p.kwargs = None;
         let named = [Value::new_none()];
         p.named = &named;
-        let names = [(Symbol::new("test"), heap.alloc_string_value("test"))];
+        let names = [(Symbol::new("test"), heap.alloc_str("test"))];
         p.names = &names;
         assert!(p.no_named_args().is_err());
     }
