@@ -36,22 +36,40 @@ use once_cell::sync::Lazy;
 use crate::{
     codemap::{CodeMap, FileSpan, Span},
     errors::Frame,
-    values::{ControlError, FrozenRef, Trace, Tracer, Value},
+    values::{error::ControlError, FrozenRef, Trace, Tracer, Value},
 };
 
 #[derive(Debug, Clone, Copy, Dupe)]
 pub(crate) struct FrozenFileSpan {
-    pub(crate) file: FrozenRef<'static, CodeMap>,
-    pub(crate) span: Span,
+    file: FrozenRef<'static, CodeMap>,
+    span: Span,
+}
+
+impl FrozenFileSpan {
+    pub(crate) const fn new_unchecked(file: FrozenRef<'static, CodeMap>, span: Span) -> Self {
+        FrozenFileSpan { file, span }
+    }
+
+    pub(crate) fn new(file: FrozenRef<'static, CodeMap>, span: Span) -> Self {
+        // Check the span is valid: this will panic if the span is not valid.
+        file.source_span(span);
+
+        Self::new_unchecked(file, span)
+    }
+
+    pub(crate) fn file(&self) -> FrozenRef<'static, CodeMap> {
+        self.file
+    }
+
+    pub(crate) fn span(&self) -> Span {
+        self.span
+    }
 }
 
 impl Default for FrozenFileSpan {
     fn default() -> Self {
         static EMPTY_FILE: Lazy<CodeMap> = Lazy::new(CodeMap::default);
-        FrozenFileSpan {
-            file: FrozenRef::new(&EMPTY_FILE),
-            span: Span::default(),
-        }
+        FrozenFileSpan::new(FrozenRef::new(&EMPTY_FILE), Span::default())
     }
 }
 
@@ -98,7 +116,7 @@ impl CheapFrame<'_> {
 
     fn to_frame(&self) -> Frame {
         Frame {
-            name: self.function.to_repr(),
+            name: self.function.name_for_call_stack(),
             location: self.location(),
         }
     }
@@ -115,12 +133,12 @@ impl Debug for CheapFrame<'_> {
 
 /// Starlark call stack.
 #[derive(Debug)]
-pub(crate) struct CallStack<'v> {
+pub(crate) struct CheapCallStack<'v> {
     count: usize,
     stack: [CheapFrame<'v>; MAX_CALLSTACK_RECURSION],
 }
 
-impl<'v> Default for CallStack<'v> {
+impl<'v> Default for CheapCallStack<'v> {
     fn default() -> Self {
         Self {
             count: 0,
@@ -136,21 +154,22 @@ impl<'v> Default for CallStack<'v> {
 // low...)
 const MAX_CALLSTACK_RECURSION: usize = 40;
 
-unsafe impl<'v> Trace<'v> for CallStack<'v> {
+unsafe impl<'v> Trace<'v> for CheapCallStack<'v> {
     fn trace(&mut self, tracer: &Tracer<'v>) {
-        for x in self.stack[0..self.count].iter_mut() {
+        let (used, unused) = self.stack.split_at_mut(self.count);
+        for x in used {
             x.function.trace(tracer);
         }
         // Not required, but since we are chosing not to walk those above
         // the current stack depth, it's good practice to blank those values out
-        for x in self.stack[self.count..].iter_mut() {
+        for x in unused {
             x.function = Value::new_none();
             x.span = None;
         }
     }
 }
 
-impl<'v> CallStack<'v> {
+impl<'v> CheapCallStack<'v> {
     /// Push an element to the stack. It is important the each `push` is paired
     /// with a `pop`.
     pub(crate) fn push(
@@ -184,9 +203,10 @@ impl<'v> CallStack<'v> {
         }
     }
 
-    pub(crate) fn to_diagnostic_frames(&self) -> Vec<Frame> {
+    pub(crate) fn to_diagnostic_frames(&self) -> CallStack {
         // The first entry is just the entire module, so skip it
-        self.stack[1..self.count].map(CheapFrame::to_frame)
+        let frames = self.stack[1..self.count].map(CheapFrame::to_frame);
+        CallStack { frames }
     }
 
     /// List the entries on the stack as values
@@ -194,3 +214,37 @@ impl<'v> CallStack<'v> {
         self.stack[1..self.count].map(|x| x.function)
     }
 }
+
+/// Owned call stack.
+#[derive(Debug, Default, Clone, PartialEq, Eq, Hash)]
+pub struct CallStack {
+    frames: Vec<Frame>,
+}
+
+impl CallStack {
+    /// Is the call stack empty?
+    pub fn is_empty(&self) -> bool {
+        self.frames.is_empty()
+    }
+
+    /// Take the contained frames.
+    pub fn into_frames(self) -> Vec<Frame> {
+        self.frames
+    }
+}
+
+impl Display for CallStack {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        if !self.frames.is_empty() {
+            // Match Python output.
+            writeln!(f, "Traceback (most recent call last):")?;
+            // TODO(nga): use real module name.
+            let mut prev = "<module>";
+            for x in &self.frames {
+                x.write_two_lines("  ", prev, f)?;
+                prev = &x.name;
+            }
+        }
+        Ok(())
+    }
+}
