@@ -29,29 +29,29 @@ use crate::{
     codemap::{FileSpan, FileSpanRef},
     collections::{alloca::Alloca, string_pool::StringPool},
     environment::{slots::ModuleSlotId, EnvironmentError, FrozenModuleRef, Module},
-    errors::{Diagnostic, Frame},
+    errors::Diagnostic,
     eval::{
         bc::frame::BcFramePtr,
-        fragment::def::DefInfo,
+        compiler::def::DefInfo,
         runtime::{
             bc_profile::BcProfile,
             before_stmt::BeforeStmt,
-            call_stack::{CallStack, FrozenFileSpan},
+            call_stack::{CheapCallStack, FrozenFileSpan},
             flame_profile::FlameProfile,
             heap_profile::{HeapProfile, HeapProfileFormat},
             profile::ProfileMode,
             slots::LocalSlotId,
             stmt_profile::StmtProfile,
         },
-        FileLoader,
+        CallStack, FileLoader,
     },
     stdlib::{
         breakpoint::{BreakpointConsole, RealBreakpointConsole},
         extra::{PrintHandler, StderrPrintHandler},
     },
     values::{
-        value_captured_get, FrozenHeap, FrozenRef, Heap, Trace, Tracer, Value, ValueCaptured,
-        ValueLike,
+        layout::value_captured::{value_captured_get, ValueCaptured},
+        FrozenHeap, FrozenRef, Heap, Trace, Tracer, Value, ValueLike,
     },
 };
 
@@ -120,7 +120,7 @@ pub struct Evaluator<'v, 'a> {
     pub(crate) print_handler: &'a (dyn PrintHandler + 'a),
     // The Starlark-level call-stack of functions.
     // Must go last because it's quite a big structure
-    pub(crate) call_stack: CallStack<'v>,
+    pub(crate) call_stack: CheapCallStack<'v>,
 }
 
 unsafe impl<'v> Trace<'v> for Evaluator<'v, '_> {
@@ -140,7 +140,7 @@ impl<'v, 'a> Evaluator<'v, 'a> {
     /// [`set_loader`](Evaluator::set_loader).
     pub fn new(module: &'v Module) -> Self {
         Evaluator {
-            call_stack: CallStack::default(),
+            call_stack: CheapCallStack::default(),
             module_env: module,
             module_variables: None,
             current_frame: BcFramePtr::null(),
@@ -278,7 +278,7 @@ impl<'v, 'a> Evaluator<'v, 'a> {
     }
 
     /// Obtain the current call-stack, suitable for use with [`Diagnostic`].
-    pub fn call_stack(&self) -> Vec<Frame> {
+    pub fn call_stack(&self) -> CallStack {
         self.call_stack.to_diagnostic_frames()
     }
 
