@@ -34,9 +34,7 @@ use gazebo::{any::AnyLifetime, prelude::*};
 use crate as starlark;
 use crate::{
     eval::runtime::csv::CsvWriter,
-    values::{
-        Freeze, Freezer, Heap, NoSimpleValue, SimpleValue, StarlarkValue, Trace, Value, ValueLike,
-    },
+    values::{Freeze, Freezer, Heap, NoSimpleValue, StarlarkValue, Trace, Value, ValueLike},
 };
 
 #[derive(Copy, Clone, Dupe, Debug)]
@@ -70,18 +68,14 @@ struct NoDrop;
 impl MaybeDrop for NeedsDrop {}
 impl MaybeDrop for NoDrop {}
 
-#[derive(Trace, Debug, Display)]
+#[derive(Trace, Debug, Display, AnyLifetime, NoSerialize)]
 #[display(fmt = "CallEnter")]
-struct CallEnter<'v, D: MaybeDrop> {
+struct CallEnter<'v, D: MaybeDrop + 'static> {
     function: Value<'v>,
     time: Instant,
     maybe_drop: D,
 }
 
-unsafe impl<'v, D: MaybeDrop + AnyLifetime<'v>> AnyLifetime<'v> for CallEnter<'v, D> {
-    any_lifetime_body!(CallEnter<'static, D>);
-}
-
 impl<'v, D: MaybeDrop + AnyLifetime<'v> + Trace<'v>> Freeze for CallEnter<'v, D> {
     type Frozen = NoSimpleValue;
     fn freeze(self, _freezer: &Freezer) -> anyhow::Result<Self::Frozen> {
@@ -93,19 +87,13 @@ impl<'v, D: MaybeDrop + AnyLifetime<'v> + Trace<'v>> StarlarkValue<'v> for CallE
     starlark_type!("call_enter");
 }
 
-#[derive(Debug, Display)]
+#[derive(Debug, Display, AnyLifetime, NoSerialize)]
 #[display(fmt = "CallExit")]
-struct CallExit<D: MaybeDrop> {
+struct CallExit<D: MaybeDrop + 'static> {
     time: Instant,
     maybe_drop: D,
 }
 
-impl<D: MaybeDrop + AnyLifetime<'static>> SimpleValue for CallExit<D> {}
-
-unsafe impl<'v, D: MaybeDrop> AnyLifetime<'v> for CallExit<D> {
-    any_lifetime_body!(CallExit<D>);
-}
-
 impl<'v, D: MaybeDrop + AnyLifetime<'static>> StarlarkValue<'v> for CallExit<D> {
     starlark_type!("call_exit");
 }
@@ -173,6 +161,8 @@ impl HeapProfile {
         self.enabled = true;
     }
 
+    #[cold]
+    #[inline(never)]
     pub(crate) fn record_call_enter<'v>(&self, function: Value<'v>, heap: &'v Heap) {
         if self.enabled {
             let time = Instant::now();
@@ -191,6 +181,8 @@ impl HeapProfile {
         }
     }
 
+    #[cold]
+    #[inline(never)]
     pub(crate) fn record_call_exit<'v>(&self, heap: &'v Heap) {
         if self.enabled {
             let time = Instant::now();
@@ -244,7 +236,9 @@ impl HeapProfile {
 
     fn write_flame_heap_profile_to(mut file: impl Write, heap: &Heap) -> anyhow::Result<()> {
         let mut collector = flame::StackCollector::new();
-        heap.for_each_ordered(|x| collector.process(x));
+        unsafe {
+            heap.for_each_ordered(|x| collector.process(x));
+        }
         collector.write_to(&mut file)?;
         Ok(())
     }
@@ -262,7 +256,9 @@ impl HeapProfile {
             call_stack: vec![(root, Duration::default(), start)],
         };
         info.ensure(root);
-        heap.for_each_ordered(|x| info.process(x));
+        unsafe {
+            heap.for_each_ordered(|x| info.process(x));
+        }
         // Just has root left on it
         assert!(info.call_stack.len() == 1);
 
@@ -593,13 +589,13 @@ mod flame {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::*;
     use crate::{
         environment::{Globals, Module},
         eval::{
             runtime::heap_profile::summary::{FuncInfo, Info},
-            Evaluator,
+            Evaluator, ProfileMode,
         },
         syntax::{AstModule, Dialect},
         values::Value,
@@ -622,7 +618,7 @@ f
         let globals = Globals::standard();
         let module = Module::new();
         let mut eval = Evaluator::new(&module);
-        eval.enable_heap_profile();
+        eval.enable_profile(&ProfileMode::Heap);
         let f = eval.eval_module(ast, &globals)?;
         // first check module profiling works
         HeapProfile::write_summarized_heap_profile_to(&mut Vec::new(), module.heap())?;
@@ -631,7 +627,7 @@ f
         // second check function profiling works
         let module = Module::new();
         let mut eval = Evaluator::new(&module);
-        eval.enable_heap_profile();
+        eval.enable_profile(&ProfileMode::Heap);
         eval.eval_function(f, &[Value::new_int(100)], &[])?;
         HeapProfile::write_summarized_heap_profile_to(&mut Vec::new(), module.heap())?;
         HeapProfile::write_flame_heap_profile_to(&mut Vec::new(), module.heap())?;
@@ -640,7 +636,7 @@ f
         let module = Module::new();
         let mut eval = Evaluator::new(&module);
         module.heap().alloc("Thing that goes before");
-        eval.enable_heap_profile();
+        eval.enable_profile(&ProfileMode::Heap);
         eval.eval_function(f, &[Value::new_int(100)], &[])?;
         module.heap().alloc("Thing that goes after");
         HeapProfile::write_summarized_heap_profile_to(&mut Vec::new(), module.heap())?;
@@ -666,7 +662,7 @@ _ignore = str([1])     # allocate a string in non_drop
         let globals = Globals::standard();
         let module = Module::new();
         let mut eval = Evaluator::new(&module);
-        eval.enable_heap_profile();
+        eval.enable_profile(&ProfileMode::Heap);
 
         eval.eval_module(ast, &globals).unwrap();
 
@@ -679,7 +675,9 @@ _ignore = str([1])     # allocate a string in non_drop
             call_stack: vec![(root, Duration::ZERO, Instant::now())],
         };
 
-        eval.heap().for_each_ordered(|v| info.process(v));
+        unsafe {
+            eval.heap().for_each_ordered(|v| info.process(v));
+        }
 
         let total = FuncInfo::merge(info.info.iter());
         // from non-drop heap
