@@ -320,9 +320,9 @@ fn test_radd() {
             let lhs: Select = UnpackValue::unpack_value(lhs).unwrap();
             Some(Ok(heap.alloc(lhs.add(self))))
         }
-        fn add(&self, rhs: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        fn add(&self, rhs: Value<'v>, heap: &'v Heap) -> Option<anyhow::Result<Value<'v>>> {
             let rhs: Select = UnpackValue::unpack_value(rhs).unwrap();
-            Ok(heap.alloc(self.clone().add(&rhs)))
+            Some(Ok(heap.alloc(self.clone().add(&rhs))))
         }
         fn collect_repr(&self, collector: &mut String) {
             write!(collector, "{}", self).unwrap()
@@ -447,10 +447,15 @@ add3(8)"#,
     assert_eq!(
         &format!("\n{}", diag),
         r#"
-* assert.bzl.add3 (called from assert.bzl:5:1-8)
-* imported.bzl.add2 (called from assert.bzl:4:5-12)
-* imported.bzl.add (called from imported.bzl:9:3-9)
-* append (called from imported.bzl:11:3-14)
+Traceback (most recent call last):
+  * assert.bzl:5, in <module>
+    add3(8)
+  * assert.bzl:4, in add3
+    add2(z)
+  * imported.bzl:9, in add2
+    add(z)
+  * imported.bzl:11, in add
+    x.append(z)
 error: Immutable
   --> imported.bzl:11:3
    |
@@ -462,10 +467,15 @@ error: Immutable
     assert_eq!(
         &format!("\n{:#}", diag),
         r#"
-* assert.bzl.add3 (called from assert.bzl:5:1-8)
-* imported.bzl.add2 (called from assert.bzl:4:5-12)
-* imported.bzl.add (called from imported.bzl:9:3-9)
-* append (called from imported.bzl:11:3-14)
+Traceback (most recent call last):
+  * assert.bzl:5, in <module>
+    add3(8)
+  * assert.bzl:4, in add3
+    add2(z)
+  * imported.bzl:9, in add2
+    add(z)
+  * imported.bzl:11, in add
+    x.append(z)
 error: Immutable
   --> imported.bzl:11:3
    |
@@ -561,6 +571,10 @@ fn test_getattr_did_you_mean_custom() {
         "struct(grey=1).gray",
         "Object of type `struct` has no attribute `gray`, did you mean `grey`?",
     );
+    assert::fail(
+        "record(grey=int.type)(grey=1).gray",
+        "Object of type `record` has no attribute `gray`, did you mean `grey`?",
+    );
 }
 
 #[test]
@@ -697,8 +711,8 @@ fn test_label_assign() {
     }
 
     #[starlark_module]
-    fn module(builder: &mut GlobalsBuilder) {
-        fn wrapper() -> anyhow::Result<Value<'v>> {
+    fn module<'v>(builder: &mut GlobalsBuilder) {
+        fn wrapper<'v>() -> anyhow::Result<Value<'v>> {
             Ok(heap.alloc_complex(Wrapper(RefCell::new(SmallMap::new()))))
         }
     }
