@@ -21,13 +21,13 @@ use crate as starlark;
 use crate::{
     assert::Assert,
     environment::GlobalsBuilder,
-    eval::{Def, FrozenDef},
+    eval::compiler::def::{Def, FrozenDef},
     values::{Value, ValueLike},
 };
 
 #[starlark_module]
 fn globals(builder: &mut GlobalsBuilder) {
-    fn returns_type_is(value: Value<'v>) -> anyhow::Result<bool> {
+    fn returns_type_is<'v>(value: Value<'v>) -> anyhow::Result<bool> {
         Ok(if let Some(def) = value.downcast_ref::<FrozenDef>() {
             def.def_info.inline_def_body.is_some()
         } else if let Some(def) = value.downcast_ref::<Def>() {
