@@ -60,7 +60,7 @@ impl BreakpointConsole for RealBreakpointConsole {
 impl RealBreakpointConsole {
     pub(crate) fn factory() -> Box<dyn Fn() -> Box<dyn BreakpointConsole>> {
         box || box RealBreakpointConsole {
-            read_line: ReadLine::new(),
+            read_line: ReadLine::new("STARLARK_RUST_DEBUGGER_HISTFILE"),
         }
     }
 }
@@ -103,8 +103,8 @@ fn cmd_variables(eval: &mut Evaluator, rl: &mut dyn BreakpointConsole) -> anyhow
 }
 
 fn cmd_stack(eval: &mut Evaluator, rl: &mut dyn BreakpointConsole) -> anyhow::Result<Next> {
-    for x in eval.call_stack() {
-        rl.println(&format!("* {}", x))
+    for line in eval.call_stack().to_string().lines() {
+        rl.println(line)
     }
     Ok(Next::Again)
 }
