@@ -31,17 +31,17 @@ use crate::{
     self as starlark,
     collections::symbol_map::Symbol,
     environment::GlobalsBuilder,
-    eval::{Arguments, Evaluator},
+    eval::{runtime::rust_loc::rust_loc, Arguments, Evaluator},
     values::{
-        dict::DictRef, function::FUNCTION_TYPE, none::NoneType, tuple::Tuple, Freeze, Freezer,
-        FrozenStringValue, FrozenValue, StarlarkValue, StringValue, StringValueLike, Trace, Value,
-        ValueLike,
+        dict::DictRef, function::FUNCTION_TYPE, layout::typed::string::StringValueLike,
+        none::NoneType, regex::StarlarkRegex, tuple::Tuple, Freeze, Freezer, FrozenStringValue,
+        FrozenValue, StarlarkValue, StringValue, Trace, Value, ValueLike,
     },
 };
 
 #[starlark_module]
 pub fn filter(builder: &mut GlobalsBuilder) {
-    fn filter(ref func: Value, ref seq: Value) -> anyhow::Result<Value<'v>> {
+    fn filter<'v>(ref func: Value, ref seq: Value) -> anyhow::Result<Value<'v>> {
         let mut res = Vec::new();
 
         for v in seq.iterate(heap)? {
@@ -59,7 +59,7 @@ pub fn filter(builder: &mut GlobalsBuilder) {
 
 #[starlark_module]
 pub fn map(builder: &mut GlobalsBuilder) {
-    fn map(ref func: Value, ref seq: Value) -> anyhow::Result<Value<'v>> {
+    fn map<'v>(ref func: Value, ref seq: Value) -> anyhow::Result<Value<'v>> {
         let it = seq.iterate(heap)?;
         let mut res = Vec::with_capacity(it.size_hint().0);
         for v in it {
@@ -71,7 +71,7 @@ pub fn map(builder: &mut GlobalsBuilder) {
 
 #[starlark_module]
 pub fn partial(builder: &mut GlobalsBuilder) {
-    fn partial(
+    fn partial<'v>(
         ref func: Value,
         args: Value<'v>,
         kwargs: DictRef<'v>,
@@ -84,7 +84,7 @@ pub fn partial(builder: &mut GlobalsBuilder) {
                 (
                     // We duplicate string here.
                     // If this becomes hot, we should do better.
-                    Symbol::new_hashed(x.unpack_starlark_str().as_str_hashed()),
+                    Symbol::new_hashed(x.as_str_hashed()),
                     x,
                 )
             })
@@ -111,7 +111,7 @@ pub fn debug(builder: &mut GlobalsBuilder) {
 pub fn dedupe(builder: &mut GlobalsBuilder) {
     /// Remove duplicates in a list. Uses identity of value (pointer),
     /// rather than by equality.
-    fn dedupe(ref val: Value) -> anyhow::Result<Value<'v>> {
+    fn dedupe<'v>(ref val: Value) -> anyhow::Result<Value<'v>> {
         let mut seen = HashSet::new();
         let mut res = Vec::new();
         for v in val.iterate(heap)? {
@@ -125,6 +125,14 @@ pub fn dedupe(builder: &mut GlobalsBuilder) {
     }
 }
 
+#[starlark_module]
+pub fn regex(builder: &mut GlobalsBuilder) {
+    /// Creates a regex which can be used for matching
+    fn regex<'v>(ref regex: &str) -> anyhow::Result<StarlarkRegex> {
+        StarlarkRegex::new(regex)
+    }
+}
+
 struct PrintWrapper<'a, 'b>(&'a Vec<Value<'b>>);
 impl fmt::Display for PrintWrapper<'_, '_> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
@@ -249,6 +257,10 @@ where
 {
     starlark_type!(FUNCTION_TYPE);
 
+    fn name_for_call_stack(&self, _me: Value<'v>) -> String {
+        "partial".to_owned()
+    }
+
     fn invoke(
         &self,
         _me: Value<'v>,
@@ -271,7 +283,9 @@ where
                         args: args.args,
                         kwargs: args.kwargs,
                     };
-                    self.func.invoke(&params, eval)
+                    self.func
+                        .to_value()
+                        .invoke_with_loc(Some(rust_loc!()), &params, eval)
                 })
             })
         })
@@ -359,7 +373,7 @@ assert_eq(
         assert::pass(
             r#"assert_eq(
                 debug([1,2]),
-                "Value(ListGen(List { content: Cell { value: ValueType(Value(Array { len: 2, capacity: 2, iter_count: 0, content: [Value(1), Value(2)] })) } }))"
+                "Value(ListGen(List { content: Cell { value: ValueTyped(Value(Array { len: 2, capacity: 2, iter_count: 0, content: [Value(1), Value(2)] })) } }))"
                 )"#,
         );
     }
