@@ -18,7 +18,7 @@
 //! A module with the standard function and constants that are by default in all
 //! dialect of Starlark
 
-use std::{cmp::Ordering, num::NonZeroI32};
+use std::{cmp::Ordering, fmt::Display, num::NonZeroI32};
 
 use anyhow::anyhow;
 
@@ -234,7 +234,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// ```
     #[starlark(type(Dict::TYPE))]
     #[starlark(speculative_exec_safe)]
-    fn dict(args: &Arguments<'v, '_>) -> anyhow::Result<Dict<'v>> {
+    fn dict<'v>(args: &Arguments<'v, '_>) -> anyhow::Result<Dict<'v>> {
         // Dict is super hot, and has a slightly odd signature, so we can do a bunch of special cases on it.
         // In particular, we don't generate the kwargs if there are no positional arguments.
         // Therefore we make it take the raw Arguments.
@@ -311,7 +311,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// # "#);
     /// ```
     #[starlark(speculative_exec_safe)]
-    fn enumerate(ref it: Value, start @ 0: i32) -> anyhow::Result<Value<'v>> {
+    fn enumerate<'v>(ref it: Value, start @ 0: i32) -> anyhow::Result<Value<'v>> {
         let v = it
             .iterate(heap)?
             .enumerate()
@@ -392,7 +392,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// # "#);
     /// ```
     #[starlark(speculative_exec_safe)]
-    fn getattr(
+    fn getattr<'v>(
         ref a: Value,
         ref attr: &str,
         ref default: Option<Value>,
@@ -512,9 +512,9 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// ```
     #[starlark(type(INT_TYPE))]
     #[starlark(speculative_exec_safe)]
-    fn int(ref a: Option<Value>, base: Option<Value>) -> anyhow::Result<i32> {
+    fn int<'v>(ref a: Option<Value<'v>>, base: Option<Value<'v>>) -> anyhow::Result<Value<'v>> {
         if a.is_none() {
-            return Ok(0);
+            return Ok(Value::new_int(0));
         }
         let a = a.unwrap();
         if let Some(s) = a.unpack_str() {
@@ -528,11 +528,11 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
                     base
                 ));
             }
-            let (sign, s) = {
+            let (negate, s) = {
                 match s.chars().next() {
-                    Some('+') => (1, s.get(1..).unwrap()),
-                    Some('-') => (-1, s.get(1..).unwrap()),
-                    _ => (1, s),
+                    Some('+') => (false, s.get(1..).unwrap()),
+                    Some('-') => (true, s.get(1..).unwrap()),
+                    _ => (false, s),
                 }
             };
             let base = if base == 0 {
@@ -569,30 +569,45 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
                 }
                 _ => s,
             };
-            match i32::from_str_radix(s, base) {
-                Ok(i) => Ok(sign * i),
-                Err(x) => Err(anyhow!(
+            fn err(a: Value, base: u32, error: impl Display) -> anyhow::Error {
+                anyhow!(
                     "{} is not a valid number in base {}: {}",
                     a.to_repr(),
                     base,
-                    x,
-                )),
+                    error,
+                )
+            }
+            match (u32::from_str_radix(s, base), negate) {
+                (Ok(i), false) => i32::try_from(i)
+                    .map(Value::new_int)
+                    .map_err(|_| err(a, base, "overflow")),
+                (Ok(i), true) => {
+                    if i > 0x80000000 {
+                        Err(err(a, base, "overflow"))
+                    } else {
+                        Ok(Value::new_int(0u32.wrapping_sub(i) as i32))
+                    }
+                }
+                (Err(x), _) => Err(err(a, base, x)),
             }
         } else if let Some(base) = base {
             Err(anyhow!(
                 "int() cannot convert non-string with explicit base '{}'",
                 base.to_repr()
             ))
-        } else if let Some(Num::Float(f)) = a.unpack_num() {
-            match Num::from(f.trunc()).as_int() {
-                Some(i) => Ok(i),
-                None => Err(anyhow!(
-                    "int() cannot convert float to integer: {}",
-                    a.to_repr()
-                )),
+        } else if let Some(num) = a.unpack_num() {
+            match num {
+                Num::Float(f) => match Num::from(f.trunc()).as_int() {
+                    Some(i) => Ok(Value::new_int(i)),
+                    None => Err(anyhow!(
+                        "int() cannot convert float to integer: {}",
+                        a.to_repr()
+                    )),
+                },
+                Num::Int(..) | Num::BigInt(..) => Ok(a),
             }
         } else {
-            Ok(a.to_int()?)
+            Ok(Value::new_int(a.to_int()?))
         }
     }
 
@@ -642,7 +657,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// ```
     #[starlark(type(List::TYPE))]
     #[starlark(speculative_exec_safe)]
-    fn list(ref a: Option<Value>) -> anyhow::Result<Value<'v>> {
+    fn list<'v>(ref a: Option<Value>) -> anyhow::Result<Value<'v>> {
         Ok(if let Some(a) = a {
             if let Some(xs) = List::from_value(a) {
                 heap.alloc_list(xs.content())
@@ -674,7 +689,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// # "#);
     /// ```
     #[starlark(speculative_exec_safe)]
-    fn max(mut args: Vec<Value>, key: Option<Value>) -> anyhow::Result<Value<'v>> {
+    fn max<'v>(mut args: Vec<Value>, key: Option<Value>) -> anyhow::Result<Value<'v>> {
         let args = if args.len() == 1 {
             args.swap_remove(0)
         } else {
@@ -728,7 +743,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// # "#);
     /// ```
     #[starlark(speculative_exec_safe)]
-    fn min(mut args: Vec<Value>, key: Option<Value>) -> anyhow::Result<Value<'v>> {
+    fn min<'v>(mut args: Vec<Value>, key: Option<Value>) -> anyhow::Result<Value<'v>> {
         let args = if args.len() == 1 {
             args.swap_remove(0)
         } else {
@@ -864,7 +879,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// "#);
     /// ```
     #[starlark(speculative_exec_safe)]
-    fn repr(ref a: Value) -> anyhow::Result<StringValue<'v>> {
+    fn repr<'v>(ref a: Value) -> anyhow::Result<StringValue<'v>> {
         let mut s = eval.string_pool.alloc();
         a.collect_repr(&mut s);
         let r = eval.heap().alloc_str(&s);
@@ -888,7 +903,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// # "#);
     /// ```
     #[starlark(speculative_exec_safe)]
-    fn reversed(ref a: Value) -> anyhow::Result<Value<'v>> {
+    fn reversed<'v>(ref a: Value) -> anyhow::Result<Value<'v>> {
         let mut v: Vec<Value> = a.iterate(heap)?.collect();
         v.reverse();
         Ok(heap.alloc_list(&v))
@@ -918,7 +933,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// ```
     // This function is not spec-safe, because it may call `key` function
     // which might be not spec-safe.
-    fn sorted(
+    fn sorted<'v>(
         ref x: Value,
         key: Option<Value>,
         reverse: Option<Value>,
@@ -975,7 +990,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// ```
     #[starlark(type(STRING_TYPE))]
     #[starlark(speculative_exec_safe)]
-    fn str(ref a: Value) -> anyhow::Result<StringValue<'v>> {
+    fn str<'v>(ref a: Value) -> anyhow::Result<StringValue<'v>> {
         if let Some(a) = StringValue::new(a) {
             // Special case that can avoid reallocating, but is equivalent.
             Ok(a)
@@ -1002,7 +1017,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// ```
     #[starlark(type(Tuple::TYPE))]
     #[starlark(speculative_exec_safe)]
-    fn tuple(ref a: Option<Value>) -> anyhow::Result<Value<'v>> {
+    fn tuple<'v>(ref a: Option<Value>) -> anyhow::Result<Value<'v>> {
         let mut l = Vec::new();
         if let Some(a) = a {
             a.with_iterator(heap, |it| {
@@ -1026,8 +1041,8 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// # "#);
     /// ```
     #[starlark(speculative_exec_safe)]
-    fn r#type(ref a: Value) -> anyhow::Result<Value<'v>> {
-        Ok(a.get_type_value().unpack().to_value())
+    fn r#type<'v>(ref a: Value) -> anyhow::Result<Value<'v>> {
+        Ok(a.get_type_value().to_frozen_value().to_value())
     }
 
     /// [zip](
@@ -1049,7 +1064,7 @@ pub(crate) fn global_functions(builder: &mut GlobalsBuilder) {
     /// # "#);
     /// ```
     #[starlark(speculative_exec_safe)]
-    fn zip(args: Vec<Value>) -> anyhow::Result<Value<'v>> {
+    fn zip<'v>(args: Vec<Value>) -> anyhow::Result<Value<'v>> {
         let mut v = Vec::new();
         let mut first = true;
         for arg in args {
@@ -1118,4 +1133,14 @@ hash(foo)
             "doesn't match",
         );
     }
+
+    #[test]
+    fn test_int() {
+        assert::eq("2147483647", "int('2147483647')");
+        assert::eq("-2147483647 - 1", "int('-2147483648')");
+        assert::eq("0", "int('0')");
+        assert::eq("0", "int('-0')");
+        assert::fail("int('2147483648')", "overflow");
+        assert::fail("int('-2147483649')", "overflow");
+    }
 }
