@@ -62,6 +62,8 @@ pub enum LibraryExtension {
     Partial,
     /// Remove duplicate entries in the list, using pointer-based equality always.
     Dedupe,
+    /// Create a regex from a string.
+    Regex,
     /// Add a function `debug(x)` which shows the Rust [`Debug`](std::fmt::Debug) representation of a value.
     /// Useful when debugging, but the output should not be considered stable.
     Debug,
@@ -83,8 +85,8 @@ impl LibraryExtension {
     pub fn all() -> &'static [Self] {
         use LibraryExtension::*;
         &[
-            StructType, RecordType, EnumType, Map, Filter, Partial, Dedupe, Debug, Print, Pprint,
-            Breakpoint, Json, Abs,
+            StructType, RecordType, EnumType, Map, Filter, Partial, Dedupe, Regex, Debug, Print,
+            Pprint, Breakpoint, Json, Abs,
         ]
     }
 
@@ -99,6 +101,7 @@ impl LibraryExtension {
             Filter => extra::filter(builder),
             Partial => extra::partial(builder),
             Dedupe => extra::dedupe(builder),
+            Regex => extra::regex(builder),
             Debug => extra::debug(builder),
             Print => extra::print(builder),
             Pprint => extra::pprint(builder),
