@@ -26,7 +26,7 @@ use crate::{
 #[starlark_module]
 pub fn global(builder: &mut GlobalsBuilder) {
     #[starlark(type(Struct::TYPE))]
-    fn r#struct(args: &Arguments<'v, '_>) -> anyhow::Result<Struct<'v>> {
+    fn r#struct<'v>(args: &Arguments<'v, '_>) -> anyhow::Result<Struct<'v>> {
         args.no_positional_args(heap)?;
         Ok(Struct::new(args.names_map()?))
     }
