@@ -17,7 +17,10 @@
 
 use std::{char, collections::VecDeque, fmt, fmt::Display};
 
+use derive_more::Display;
 use logos::Logos;
+use num_bigint::BigInt;
+use num_traits::Num;
 use thiserror::Error;
 
 use crate::{
@@ -45,8 +48,8 @@ pub(crate) enum LexemeError {
     ReservedKeyword(String),
     #[error("Parse error: integer cannot have leading 0, got `{0}`")]
     StartsZero(String),
-    #[error("Parse error: integer overflow, must fit in 32 bits, got `{0}`")]
-    IntOverflow(String),
+    #[error("Parse error: failed to parse integer: `{0}`")]
+    IntParse(String),
 }
 
 type Lexeme = anyhow::Result<(usize, Token, usize)>;
@@ -375,6 +378,20 @@ impl<'a> Lexer<'a> {
         )
     }
 
+    fn int(&self, s: &str, radix: u32) -> Lexeme {
+        let span = self.lexer.span();
+        match i32::from_str_radix(s, radix) {
+            Ok(i) => Ok((span.start, Token::Int(TokenInt::I32(i)), span.end)),
+            Err(_) => match BigInt::from_str_radix(s, radix) {
+                Ok(i) => Ok((span.start, Token::Int(TokenInt::BigInt(i)), span.end)),
+                Err(_) => {
+                    // This code is unreachable, but safer to return an error than to panic.
+                    self.err_now(LexemeError::IntParse)
+                }
+            },
+        }
+    }
+
     pub fn next(&mut self) -> Option<Lexeme> {
         loop {
             // Note that this function doesn't always return - a few branches use `continue`
@@ -416,27 +433,29 @@ impl<'a> Lexer<'a> {
                         }
                         Token::Reserved => Some(self.err_now(LexemeError::ReservedKeyword)),
                         Token::Error => Some(self.err_now(LexemeError::InvalidInput)),
-                        Token::Int(radix) => {
-                            let mut s = self.lexer.slice();
-                            if radix == 10 {
-                                if s.len() > 1 && &s[0..1] == "0" {
-                                    return Some(self.err_now(LexemeError::StartsZero));
-                                }
-                            } else {
-                                // Skip the 0x prefix
-                                s = &s[2..];
-                            }
-                            match i32::from_str_radix(s, radix as u32) {
-                                Ok(i) => {
-                                    let span = self.lexer.span();
-                                    Some(Ok((span.start, Token::Int(i), span.end)))
-                                }
-                                Err(_) => {
-                                    // Because we validated the characters going in, it must have been an overflow
-                                    Some(self.err_now(LexemeError::IntOverflow))
-                                }
+                        Token::RawDecInt => {
+                            let s = self.lexer.slice();
+                            if s.len() > 1 && &s[0..1] == "0" {
+                                return Some(self.err_now(LexemeError::StartsZero));
                             }
+                            Some(self.int(s, 10))
                         }
+                        Token::RawOctInt => {
+                            let s = self.lexer.slice();
+                            assert!(s.starts_with("0o") || s.starts_with("0O"));
+                            Some(self.int(&s[2..], 8))
+                        }
+                        Token::RawHexInt => {
+                            let s = self.lexer.slice();
+                            assert!(s.starts_with("0x") || s.starts_with("0X"));
+                            Some(self.int(&s[2..], 16))
+                        }
+                        Token::RawBinInt => {
+                            let s = self.lexer.slice();
+                            assert!(s.starts_with("0b") || s.starts_with("0B"));
+                            Some(self.int(&s[2..], 2))
+                        }
+                        Token::Int(..) => unreachable!("Lexer does not produce Int tokens"),
                         Token::RawDoubleQuote => {
                             let raw = self.lexer.span().len() == 2;
                             if self.lexer.remainder().starts_with("\"\"") {
@@ -487,6 +506,14 @@ impl<'a> Lexer<'a> {
     }
 }
 
+#[derive(Debug, Clone, Eq, PartialEq, Display)]
+pub enum TokenInt {
+    #[display(fmt = "{}", .0)]
+    I32(i32),
+    #[display(fmt = "{}", .0)]
+    BigInt(BigInt),
+}
+
 /// All token that can be generated by the lexer
 #[derive(Logos, Debug, Clone, PartialEq)]
 pub enum Token {
@@ -525,11 +552,16 @@ pub enum Token {
     , |lex| lex.slice().to_owned())]
     Identifier(String), // An identifier
 
-    #[regex("[0-9]+", |_| 10)]
-    #[regex("0[xX][A-Fa-f0-9]+", |_| 16)]
-    #[regex("0[bB][01]+", |_| 2)]
-    #[regex("0[oO][0-7]+", |_| 8)]
-    Int(i32), // An integer literal (123, 0x1, 0b1011, 0o755, ...)
+    #[regex("[0-9]+")]
+    RawDecInt,
+    #[regex("0[xX][A-Fa-f0-9]+")]
+    RawHexInt,
+    #[regex("0[bB][01]+")]
+    RawBinInt,
+    #[regex("0[oO][0-7]+")]
+    RawOctInt,
+
+    Int(TokenInt), // An integer literal (123, 0x1, 0b1011, 0o755, ...)
 
     #[regex("[0-9]+\\.[0-9]*([eE][-+]?[0-9]+)?", |lex| lex.slice().parse::<f64>())]
     #[regex("[0-9]+[eE][-+]?[0-9]+", |lex| lex.slice().parse::<f64>())]
@@ -754,6 +786,10 @@ impl Display for Token {
             Token::Reserved => write!(f, "reserved keyword"),
             Token::Identifier(s) => write!(f, "identifier '{}'", s),
             Token::Int(i) => write!(f, "integer literal '{}'", i),
+            Token::RawDecInt => write!(f, "decimal integer literal"),
+            Token::RawHexInt => write!(f, "hexadecimal integer literal"),
+            Token::RawOctInt => write!(f, "octal integer literal"),
+            Token::RawBinInt => write!(f, "binary integer literal"),
             Token::Float(n) => write!(f, "float literal '{}'", n),
             Token::String(s) => write!(f, "string literal '{}'", s),
             Token::RawSingleQuote => write!(f, "starting '"),
