@@ -266,10 +266,6 @@ fn test_lexer_error_messages() {
         "an + 'invalid escape !\\x3 ! character'",
         "invalid string escape sequence `x3 `",
     );
-    f(
-        "large_int = !1238989456723879! + 8",
-        "integer overflow, must fit in 32 bits, got `1238989456723879`",
-    );
     f(
         "leading_zero = !003! + 8",
         "integer cannot have leading 0, got `003`",
