@@ -106,7 +106,9 @@ pub enum DocStringKind {
 impl DocString {
     /// Extracts the docstring from a function or module body, iff the first
     /// statement is a string literal.
-    pub fn extract_raw_starlark_docstring<P: AstPayload>(body: &AstStmtP<P>) -> Option<String> {
+    pub(crate) fn extract_raw_starlark_docstring<P: AstPayload>(
+        body: &AstStmtP<P>,
+    ) -> Option<String> {
         if let StmtP::Statements(stmts) = &body.node {
             if let Some(Spanned {
                 node:
@@ -583,12 +585,12 @@ mod tests {
     fn add_some_value(builder: &mut MethodsBuilder) {
         /// Docs for attr1
         #[starlark(attribute)]
-        fn attr1(_this: Value<'v>) -> anyhow::Result<String> {
+        fn attr1<'v>(_this: Value<'v>) -> anyhow::Result<String> {
             Ok("attr1".to_owned())
         }
 
         #[starlark(attribute)]
-        fn attr2(_this: Value<'v>) -> anyhow::Result<String> {
+        fn attr2<'v>(_this: Value<'v>) -> anyhow::Result<String> {
             Ok("attr2".to_owned())
         }
 
@@ -599,12 +601,12 @@ mod tests {
         ///
         /// # Returns
         /// The string 'func1'
-        fn func1(_this: Value<'v>, foo: String) -> anyhow::Result<String> {
+        fn func1<'v>(_this: Value<'v>, foo: String) -> anyhow::Result<String> {
             let _ignore = foo;
             Ok("func1".to_owned())
         }
 
-        fn func2(_this: Value<'v>) -> anyhow::Result<String> {
+        fn func2<'v>(_this: Value<'v>) -> anyhow::Result<String> {
             Ok("func2".to_owned())
         }
     }
