@@ -38,7 +38,7 @@ use std::{
 
 use bumpalo::Bump;
 use either::Either;
-use gazebo::prelude::*;
+use gazebo::{any::AnyLifetime, prelude::*};
 
 use crate::values::{
     layout::avalue::{AValue, AValueDyn, BlackHole},
@@ -131,12 +131,12 @@ impl AValueForward {
 /// Reservation is morally a Reservation<T>, but we treat is as an
 /// existential.
 /// Tied to the lifetime of the heap.
-pub(crate) struct Reservation<'v, 'v2, T: AValueDyn<'v2>> {
+pub(crate) struct Reservation<'v, 'v2, T: AValue<'v2>> {
     pointer: *mut AValueRepr<T>, // Secretly AValueObject<T>
     phantom: PhantomData<(&'v (), &'v2 T)>,
 }
 
-impl<'v, 'v2, T: AValueDyn<'v2>> Reservation<'v, 'v2, T> {
+impl<'v, 'v2, T: AValue<'v2>> Reservation<'v, 'v2, T> {
     pub(crate) fn fill(self, x: T) {
         unsafe {
             ptr::write(
@@ -224,7 +224,7 @@ impl Arena {
         // from `AValueHeader` (with `TAG_STR` when appropriate).
         // `BlackHole` assumes it is created for non-string, so
         // it returns `false` from `is_str`.
-        assert!(!T::is_str());
+        assert!(!T::IS_STR);
 
         let (p, extra) = Self::alloc_uninit::<T>(self.bump_for_type::<T>(), extra_len);
         // If we don't have a vtable we can't skip over missing elements to drop,
@@ -454,7 +454,7 @@ impl AValueHeader {
     /// Cast header pointer to repr pointer.
     pub(crate) unsafe fn as_repr<'v, A: AValue<'v>>(&self) -> &AValueRepr<A> {
         debug_assert_eq!(
-            A::static_type_id_of_value(),
+            A::StarlarkValue::static_type_id(),
             self.unpack().static_type_of_value()
         );
         &*(self as *const AValueHeader as *const AValueRepr<A>)
@@ -463,7 +463,7 @@ impl AValueHeader {
     /// Cast header pointer to repr pointer.
     pub(crate) unsafe fn as_repr_mut<'v, A: AValue<'v>>(&mut self) -> &mut AValueRepr<A> {
         debug_assert_eq!(
-            A::static_type_id_of_value(),
+            A::StarlarkValue::static_type_id(),
             self.unpack().static_type_of_value()
         );
         &mut *(self as *mut AValueHeader as *mut AValueRepr<A>)
