@@ -33,7 +33,7 @@ use serde::{Serialize, Serializer};
 use crate::{
     collections::{StarlarkHashValue, StarlarkHasher},
     environment::Methods,
-    eval::{Arguments, Evaluator, FrozenDef},
+    eval::{compiler::def::FrozenDef, Arguments, Evaluator},
     values::{
         basic::StarlarkValueBasic,
         bool::StarlarkBool,
@@ -44,12 +44,13 @@ use crate::{
         none::NoneType,
         num::Num,
         string::StarlarkStr,
+        traits::StarlarkValueDyn,
         types::{
             array::Array,
             tuple::{FrozenTuple, Tuple},
         },
-        ComplexValue, Freezer, FrozenStringValue, FrozenValue, Heap, StarlarkValue,
-        StarlarkValueDyn, Trace, Tracer, Value, ValueTyped,
+        ComplexValue, Freezer, FrozenStringValue, FrozenValue, Heap, StarlarkValue, Trace, Tracer,
+        Value, ValueTyped,
     },
 };
 
@@ -140,9 +141,8 @@ pub(crate) trait AValue<'v>: StarlarkValueDyn<'v> + Sized {
     /// Return `mem::size_of::<Self>()` if there's no extra content.
     fn offset_of_extra() -> usize;
 
-    fn is_str() -> bool {
-        false
-    }
+    /// Type is `StarlarkStr`.
+    const IS_STR: bool = false;
 
     fn memory_size_for_extra_len(extra_len: usize) -> usize {
         assert!(
@@ -210,7 +210,7 @@ impl<'v, A: AValue<'v>> AValueDyn<'v> for A {
     }
 
     fn is_str(&self) -> bool {
-        A::is_str()
+        A::IS_STR
     }
 
     fn get_hash(&self) -> anyhow::Result<StarlarkHashValue> {
@@ -395,9 +395,7 @@ impl<'v> AValue<'v> for AValueImpl<Direct, StarlarkStr> {
         StarlarkStr::offset_of_content()
     }
 
-    fn is_str() -> bool {
-        true
-    }
+    const IS_STR: bool = true;
 
     unsafe fn heap_freeze(
         me: *mut AValueRepr<Self>,
@@ -408,7 +406,7 @@ impl<'v> AValue<'v> for AValueImpl<Direct, StarlarkStr> {
             "short strings are allocated statically"
         );
 
-        let s = (*me).payload.1.unpack();
+        let s = (*me).payload.1.as_str();
         let fv = freezer.alloc(s);
         debug_assert!(fv.is_str());
         AValueHeader::overwrite_with_forward::<Self>(me, fv.0.ptr_value());
@@ -421,7 +419,7 @@ impl<'v> AValue<'v> for AValueImpl<Direct, StarlarkStr> {
             "short strings are allocated statically"
         );
 
-        let s = (*me).payload.1.unpack();
+        let s = (*me).payload.1.as_str();
         let v = tracer.alloc_str(s);
         debug_assert!(v.is_str());
         AValueHeader::overwrite_with_forward::<Self>(me, v.0.ptr_value() & !1);
@@ -767,13 +765,6 @@ impl<'v> AValueDyn<'v> for BlackHole {
 }
 
 impl<'v> StarlarkValueDyn<'v> for BlackHole {
-    fn static_type_id_of_value() -> TypeId
-    where
-        Self: Sized,
-    {
-        panic!()
-    }
-
     fn static_type_of_value(&self) -> TypeId {
         panic!()
     }
@@ -814,6 +805,9 @@ impl<'v> StarlarkValueDyn<'v> for BlackHole {
     fn collect_repr_cycle(&self, _collector: &mut String) {
         panic!()
     }
+    fn name_for_call_stack(&self, _me: Value<'v>) -> String {
+        panic!()
+    }
     fn to_bool(&self) -> bool {
         panic!()
     }
@@ -901,7 +895,7 @@ impl<'v> StarlarkValueDyn<'v> for BlackHole {
     fn radd(&self, _lhs: Value<'v>, _heap: &'v Heap) -> Option<anyhow::Result<Value<'v>>> {
         panic!()
     }
-    fn add(&self, _rhs: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+    fn add(&self, _rhs: Value<'v>, _heap: &'v Heap) -> Option<anyhow::Result<Value<'v>>> {
         panic!()
     }
     fn sub(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
@@ -919,19 +913,22 @@ impl<'v> StarlarkValueDyn<'v> for BlackHole {
     fn floor_div(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         panic!()
     }
-    fn bit_and(&self, _other: Value<'v>) -> anyhow::Result<Value<'v>> {
+    fn bit_and(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         panic!()
     }
     fn bit_or(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         panic!()
     }
-    fn bit_xor(&self, _other: Value<'v>) -> anyhow::Result<Value<'v>> {
+    fn bit_xor(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         panic!()
     }
-    fn left_shift(&self, _other: Value<'v>) -> anyhow::Result<Value<'v>> {
+    fn bit_not(&self, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         panic!()
     }
-    fn right_shift(&self, _other: Value<'v>) -> anyhow::Result<Value<'v>> {
+    fn left_shift(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        panic!()
+    }
+    fn right_shift(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         panic!()
     }
     fn export_as(&self, _variable_name: &str, _eval: &mut Evaluator<'v, '_>) {
@@ -955,12 +952,6 @@ impl Serialize for BlackHole {
 }
 
 impl<'v, Mode: 'static, T: StarlarkValue<'v>> StarlarkValueDyn<'v> for AValueImpl<Mode, T> {
-    fn static_type_id_of_value() -> TypeId
-    where
-        Self: Sized,
-    {
-        T::static_type_id()
-    }
     fn static_type_of_value(&self) -> TypeId {
         T::static_type_id()
     }
@@ -997,6 +988,9 @@ impl<'v, Mode: 'static, T: StarlarkValue<'v>> StarlarkValueDyn<'v> for AValueImp
     fn collect_repr_cycle(&self, collector: &mut String) {
         self.1.collect_repr_cycle(collector)
     }
+    fn name_for_call_stack(&self, me: Value<'v>) -> String {
+        self.1.name_for_call_stack(me)
+    }
     fn to_bool(&self) -> bool {
         self.1.to_bool()
     }
@@ -1084,7 +1078,7 @@ impl<'v, Mode: 'static, T: StarlarkValue<'v>> StarlarkValueDyn<'v> for AValueImp
     fn radd(&self, lhs: Value<'v>, heap: &'v Heap) -> Option<anyhow::Result<Value<'v>>> {
         self.1.radd(lhs, heap)
     }
-    fn add(&self, rhs: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+    fn add(&self, rhs: Value<'v>, heap: &'v Heap) -> Option<anyhow::Result<Value<'v>>> {
         self.1.add(rhs, heap)
     }
     fn sub(&self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
@@ -1102,20 +1096,23 @@ impl<'v, Mode: 'static, T: StarlarkValue<'v>> StarlarkValueDyn<'v> for AValueImp
     fn floor_div(&self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         self.1.floor_div(other, heap)
     }
-    fn bit_and(&self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
-        self.1.bit_and(other)
+    fn bit_and(&self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        self.1.bit_and(other, heap)
     }
     fn bit_or(&self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         self.1.bit_or(other, heap)
     }
-    fn bit_xor(&self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
-        self.1.bit_xor(other)
+    fn bit_xor(&self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        self.1.bit_xor(other, heap)
     }
-    fn left_shift(&self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
-        self.1.left_shift(other)
+    fn bit_not(&self, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        self.1.bit_not(heap)
     }
-    fn right_shift(&self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
-        self.1.right_shift(other)
+    fn left_shift(&self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        self.1.left_shift(other, heap)
+    }
+    fn right_shift(&self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        self.1.right_shift(other, heap)
     }
     fn export_as(&self, variable_name: &str, eval: &mut Evaluator<'v, '_>) {
         self.1.export_as(variable_name, eval)
@@ -1147,7 +1144,6 @@ mod tests {
         let list = module.heap().alloc_list(&[]);
         let tuple = module.heap().alloc_tuple(&[list]);
         List::from_value_mut(list)
-            .unwrap()
             .unwrap()
             .push(tuple, module.heap());
         module.set("t", tuple);
