@@ -36,11 +36,10 @@ use gazebo::{cast, prelude::*};
 use once_cell::sync::Lazy;
 
 use crate::{
-    eval::FrozenDef,
+    eval::compiler::def::FrozenDef,
     values::{
         any::StarlarkAny,
         array::Array,
-        constant_string,
         layout::{
             arena::{AValueHeader, AValueRepr, Arena, HeapSummary, Reservation},
             avalue::{
@@ -49,11 +48,13 @@ use crate::{
                 VALUE_EMPTY_FROZEN_LIST, VALUE_EMPTY_TUPLE,
             },
             fast_cell::FastCell,
+            static_string::constant_string,
+            typed::string::StringValueLike,
             value::{FrozenValue, Value},
         },
         types::float::StarlarkFloat,
         AllocFrozenValue, ComplexValue, FrozenRef, FrozenStringValue, FrozenValueTyped,
-        StarlarkValue, StringValue, StringValueLike, ValueTyped,
+        StarlarkValue, StringValue, ValueTyped,
     },
 };
 
@@ -565,9 +566,7 @@ impl Heap {
             // Otherwise the Value is constrainted by the borrow_mut, when
             // we consider values to be kept alive permanently, other than
             // when a GC happens
-            f(Value::new_ptr_query_is_str(unsafe {
-                cast::ptr_lifetime(x)
-            }))
+            f(Value::new_ptr_query_is_str(cast::ptr_lifetime(x)))
         })
     }
 
@@ -629,7 +628,7 @@ impl<'v> Tracer<'v> {
         Reservation<'a, 'v2, T>,
         &'a mut [MaybeUninit<T::ExtraElem>],
     ) {
-        assert!(!T::is_str(), "strings cannot be reserved");
+        assert!(!T::IS_STR, "strings cannot be reserved");
         let (r, extra) = self.arena.reserve_with_extra::<T>(extra_len);
         let v = Value::new_ptr(unsafe { cast::ptr_lifetime(r.ptr()) }, false);
         (v, r, extra)
