@@ -18,20 +18,13 @@
 // Possible optimisations:
 // Encoding none, bool etc in the pointer of frozen value
 
-pub use heap::{Freezer, FrozenHeap, FrozenHeapRef, Heap, Tracer};
-pub(crate) use pointer_i32::PointerI32;
-pub(crate) use string::StringValueLike;
-pub use string::{static_string::*, FrozenStringValue, StringValue};
-pub(crate) use value_captured::*;
-
 mod arena;
 mod avalue;
 mod fast_cell;
-mod heap;
+pub(crate) mod heap;
 pub(crate) mod identity;
 mod pointer;
-mod pointer_i32;
-mod string;
+pub(crate) mod static_string;
 pub(crate) mod typed;
 pub(crate) mod value;
-mod value_captured;
+pub(crate) mod value_captured;
