@@ -30,6 +30,7 @@
 // somewhere. The solution is to have a separate value storage vs vtable.
 
 use std::{
+    borrow::Cow,
     cmp::Ordering,
     fmt,
     fmt::{Debug, Display},
@@ -43,18 +44,19 @@ use gazebo::{
     prelude::*,
 };
 use indexmap::Equivalent;
+use num_bigint::BigInt;
 use serde::{Serialize, Serializer};
 
 use crate::{
     collections::{Hashed, StarlarkHashValue, StarlarkHasher},
-    eval::{runtime::call_stack::FrozenFileSpan, Arguments, Evaluator, FrozenDef},
+    eval::{compiler::def::FrozenDef, runtime::call_stack::FrozenFileSpan, Arguments, Evaluator},
     values::{
         dict::FrozenDict,
         docs::DocItem,
         enumeration::{EnumType, FrozenEnumValue},
         float::StarlarkFloat,
         function::{FrozenBoundMethod, NativeFunction, FUNCTION_TYPE},
-        identity::ValueIdentity,
+        int::PointerI32,
         layout::{
             arena::{AValueHeader, AValueRepr},
             avalue::{
@@ -62,7 +64,8 @@ use crate::{
                 VALUE_TRUE,
             },
             pointer::{FrozenPointer, Pointer},
-            pointer_i32::PointerI32,
+            static_string::VALUE_EMPTY_STRING,
+            typed::string::StringValueLike,
         },
         list::FrozenList,
         num::Num,
@@ -73,9 +76,9 @@ use crate::{
         string::StarlarkStr,
         structs::FrozenStruct,
         tuple::FrozenTuple,
-        unbound::MaybeUnboundValue,
+        types::unbound::MaybeUnboundValue,
         Freeze, Freezer, FrozenRef, FrozenStringValue, FrozenValueTyped, Heap, StarlarkValue,
-        StringValue, StringValueLike, UnpackValue, ValueError, VALUE_EMPTY_STRING,
+        StringValue, UnpackValue, ValueError, ValueIdentity,
     },
 };
 
@@ -198,7 +201,7 @@ impl<'v> Value<'v> {
     }
 
     pub(crate) fn new_repr<T: AValue<'v>>(x: &'v AValueRepr<T>) -> Self {
-        Self::new_ptr(&x.header, T::is_str())
+        Self::new_ptr(&x.header, T::IS_STR)
     }
 
     pub(crate) fn new_ptr_usize_with_str_tag(x: usize) -> Self {
@@ -251,10 +254,19 @@ impl<'v> Value<'v> {
     }
 
     /// Obtain the underlying numerical value, if it is one.
-    pub fn unpack_num(self) -> Option<Num> {
+    pub fn unpack_num(self) -> Option<Num<'v>> {
         Num::unpack_value(self)
     }
 
+    /// This operation allocates `BigInt` for small int, use carefully.
+    pub(crate) fn unpack_int_or_big(self) -> Option<Cow<'v, BigInt>> {
+        match self.unpack_num()? {
+            Num::Float(_) => None,
+            Num::Int(x) => Some(Cow::Owned(BigInt::from(x))),
+            Num::BigInt(x) => Some(Cow::Borrowed(x.get())),
+        }
+    }
+
     /// Obtain the underlying `bool` if it is a boolean.
     pub fn unpack_bool(self) -> Option<bool> {
         let p = self.0.ptr_value();
@@ -301,20 +313,20 @@ impl<'v> Value<'v> {
 
     /// Obtain the underlying `str` if it is a string.
     pub fn unpack_str(self) -> Option<&'v str> {
-        self.unpack_starlark_str().map(|s| s.unpack())
+        self.unpack_starlark_str().map(|s| s.as_str())
     }
 
     /// Get a pointer to a [`AValue`].
     pub(crate) fn get_ref(self) -> &'v dyn AValueDyn<'v> {
         match self.0.unpack() {
             Either::Left(x) => x.unpack(),
-            Either::Right(x) => basic_ref(PointerI32::new(x)),
+            Either::Right(x) => basic_ref(x),
         }
     }
 
     /// Downcast without checking the value type.
     pub(crate) unsafe fn downcast_ref_unchecked<T: StarlarkValue<'v>>(self) -> &'v T {
-        debug_assert!(self.downcast_ref::<T>().is_some());
+        debug_assert!(self.get_ref().downcast_ref::<T>().is_some());
         if PointerI32::type_is_pointer_i32::<T>() {
             transmute!(
                 &PointerI32,
@@ -339,7 +351,7 @@ impl<'v> Value<'v> {
     /// Note that other properties are not guaranteed, and the result is not considered part of the API.
     /// The result can be impacted by optimisations such as hash-consing, copy-on-write, partial
     /// evaluation etc.
-    pub fn ptr_eq(self, other: Self) -> bool {
+    pub fn ptr_eq(self, other: Value) -> bool {
         self.0.ptr_eq(other.0)
     }
 
@@ -450,8 +462,8 @@ impl<'v> Value<'v> {
     }
 
     /// `x & other`.
-    pub fn bit_and(self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
-        self.get_ref().bit_and(other)
+    pub fn bit_and(self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        self.get_ref().bit_and(other, heap)
     }
 
     /// `x | other`.
@@ -460,18 +472,23 @@ impl<'v> Value<'v> {
     }
 
     /// `x ^ other`.
-    pub fn bit_xor(self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
-        self.get_ref().bit_xor(other)
+    pub fn bit_xor(self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        self.get_ref().bit_xor(other, heap)
+    }
+
+    /// `~x`.
+    pub fn bit_not(self, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        self.get_ref().bit_not(heap)
     }
 
     /// `x << other`.
-    pub fn left_shift(self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
-        self.get_ref().left_shift(other)
+    pub fn left_shift(self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        self.get_ref().left_shift(other, heap)
     }
 
     /// `x >> other`.
-    pub fn right_shift(self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
-        self.get_ref().right_shift(other)
+    pub fn right_shift(self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        self.get_ref().right_shift(other, heap)
     }
 
     pub(crate) fn invoke_with_loc(
@@ -527,11 +544,35 @@ impl<'v> Value<'v> {
     /// Add two [`Value`]s together. Will first try using [`radd`](StarlarkValue::radd),
     /// before falling back to [`add`](StarlarkValue::add).
     pub fn add(self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
-        let me = self.to_value();
-        if let Some(v) = other.get_ref().radd(me, heap) {
+        // Fast special case for ints.
+        if let Some(ls) = self.unpack_int() {
+            if let Some(rs) = other.unpack_int() {
+                // On overflow take the slow path below.
+                if let Some(sum) = ls.checked_add(rs) {
+                    return Ok(Value::new_int(sum));
+                }
+            }
+        }
+
+        // Addition of string is super common and pretty cheap, so have a special case for it.
+        if let Some(ls) = self.unpack_str() {
+            if let Some(rs) = other.unpack_str() {
+                if ls.is_empty() {
+                    return Ok(other);
+                } else if rs.is_empty() {
+                    return Ok(self);
+                } else {
+                    return Ok(heap.alloc_str_concat(ls, rs).to_value());
+                }
+            }
+        }
+
+        if let Some(v) = self.get_ref().add(other, heap) {
+            v
+        } else if let Some(v) = other.get_ref().radd(self, heap) {
             v
         } else {
-            self.get_ref().add(other, heap)
+            ValueError::unsupported_owned(self.get_type(), "+", Some(other.get_type()))
         }
     }
 
@@ -556,6 +597,10 @@ impl<'v> Value<'v> {
         s
     }
 
+    pub(crate) fn name_for_call_stack(self) -> String {
+        self.get_ref().name_for_call_stack(self)
+    }
+
     /// Convert the value to JSON.
     ///
     /// Return an error if the value or any contained value does not support conversion to JSON.
@@ -699,13 +744,17 @@ impl FrozenValue {
     }
 
     pub(crate) fn new_repr<'a, T: AValue<'a>>(x: &'static AValueRepr<T>) -> Self {
-        Self::new_ptr(&x.header, T::is_str())
+        Self::new_ptr(&x.header, T::IS_STR)
     }
 
     pub(crate) fn new_ptr_usize_with_str_tag(x: usize) -> Self {
         Self(FrozenPointer::new_frozen_usize_with_str_tag(x))
     }
 
+    pub(crate) fn new_ptr_value(x: usize) -> Self {
+        unsafe { Self(FrozenPointer::new(x)) }
+    }
+
     /// Create a new value representing `None` in Starlark.
     pub fn new_none() -> Self {
         Self::new_repr(&VALUE_NONE)
@@ -730,6 +779,10 @@ impl FrozenValue {
         VALUE_EMPTY_STRING.unpack()
     }
 
+    pub(crate) fn ptr_value(self) -> usize {
+        self.0.ptr_value()
+    }
+
     /// Is a value a Starlark `None`.
     pub fn is_none(self) -> bool {
         // Safe because frozen values never have a tag
@@ -753,6 +806,10 @@ impl FrozenValue {
         self.0.unpack_int()
     }
 
+    pub(crate) unsafe fn unpack_int_unchecked(self) -> i32 {
+        self.0.unpack_int_unchecked()
+    }
+
     pub(crate) fn is_str(self) -> bool {
         self.to_value().is_str()
     }
@@ -770,7 +827,7 @@ impl FrozenValue {
     pub(crate) fn get_ref<'v>(self) -> &'v dyn AValueDyn<'v> {
         match self.0.unpack() {
             Either::Left(x) => x.unpack(),
-            Either::Right(x) => basic_ref(PointerI32::new(x)),
+            Either::Right(x) => basic_ref(x),
         }
     }
 
@@ -888,7 +945,12 @@ pub trait ValueLike<'v>:
 
     /// Get hash value.
     fn get_hashed(self) -> anyhow::Result<Hashed<Self>> {
-        Ok(Hashed::new_unchecked(self.to_value().get_hash()?, self))
+        let hash = if let Some(s) = self.to_value().unpack_starlark_str() {
+            s.get_hash()
+        } else {
+            self.to_value().get_hash()?
+        };
+        Ok(Hashed::new_unchecked(hash, self))
     }
 
     /// `repr(x)`.
@@ -928,7 +990,23 @@ impl<'v> ValueLike<'v> for Value<'v> {
     }
 
     fn downcast_ref<T: StarlarkValue<'v>>(self) -> Option<&'v T> {
-        self.get_ref().downcast_ref::<T>()
+        if T::static_type_id() == StarlarkStr::static_type_id() {
+            if self.is_str() {
+                // SAFETY: we just checked this is string, and requested type is string.
+                Some(unsafe { self.downcast_ref_unchecked() })
+            } else {
+                None
+            }
+        } else if PointerI32::type_is_pointer_i32::<T>() {
+            if self.unpack_int().is_some() {
+                // SAFETY: we just checked this is int, and requested type is int.
+                Some(unsafe { self.downcast_ref_unchecked() })
+            } else {
+                None
+            }
+        } else {
+            self.get_ref().downcast_ref::<T>()
+        }
     }
 
     fn collect_repr(self, collector: &mut String) {
@@ -1002,3 +1080,33 @@ where
     FrozenValue: Send + Sync,
 {
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::values::{
+        none::NoneType, string::StarlarkStr, types::int::PointerI32, Heap, Value, ValueLike,
+    };
+
+    #[test]
+    fn test_downcast_ref() {
+        let heap = Heap::new();
+        let string = heap.alloc_str("asd").to_value();
+        let none = Value::new_none();
+        let integer = Value::new_int(17);
+
+        assert!(string.downcast_ref::<NoneType>().is_none());
+        assert!(integer.downcast_ref::<NoneType>().is_none());
+        assert!(none.downcast_ref::<NoneType>().is_some());
+
+        assert_eq!(
+            "asd",
+            string.downcast_ref::<StarlarkStr>().unwrap().as_str()
+        );
+        assert!(integer.downcast_ref::<StarlarkStr>().is_none());
+        assert!(none.downcast_ref::<StarlarkStr>().is_none());
+
+        assert!(string.downcast_ref::<PointerI32>().is_none());
+        assert_eq!(17, integer.downcast_ref::<PointerI32>().unwrap().get());
+        assert!(none.downcast_ref::<PointerI32>().is_none());
+    }
+}
