@@ -37,17 +37,28 @@ pub use gazebo::{
 pub use starlark_derive::{starlark_attrs, Freeze, NoSerialize, StarlarkAttrs, Trace};
 
 pub use crate::values::{
-    alloc_value::*,
-    error::*,
-    freeze::*,
-    frozen_ref::*,
-    layout::{identity::*, value::*, *},
-    owned::*,
-    trace::*,
-    traits::*,
-    typed::*,
-    types::*,
-    unpack::*,
+    alloc_value::{AllocFrozenValue, AllocValue},
+    error::ValueError,
+    freeze::Freeze,
+    frozen_ref::FrozenRef,
+    layout::{
+        heap::{Freezer, FrozenHeap, FrozenHeapRef, Heap, Tracer},
+        identity::ValueIdentity,
+        static_string::StarlarkStrNRepr,
+        typed::{
+            string::{FrozenStringValue, StringValue},
+            FrozenValueTyped, ValueTyped,
+        },
+        value::{FrozenValue, Value, ValueLike},
+    },
+    owned::{OwnedFrozenValue, OwnedFrozenValueTyped},
+    trace::Trace,
+    traits::{ComplexValue, NoSimpleValue, StarlarkValue},
+    types::{
+        any, array, bool, dict, enumeration, float, function, int, list, none, range, record,
+        regex, string, structs, tuple,
+    },
+    unpack::{UnpackValue, ValueOf},
 };
 
 #[macro_use]
@@ -58,9 +69,9 @@ mod alloc_value;
 pub(crate) mod basic;
 pub mod display;
 pub mod docs;
-mod error;
+pub(crate) mod error;
 mod freeze;
-mod frozen_ref;
+pub(crate) mod frozen_ref;
 mod index;
 pub(crate) mod iter;
 pub(crate) mod layout;
