@@ -44,8 +44,8 @@ use crate::{
     environment::Methods,
     eval::{Arguments, Evaluator},
     values::{
-        docs::DocItem, function::FUNCTION_TYPE, ControlError, Freeze, FrozenStringValue, Heap,
-        Trace, Value, ValueError,
+        docs::DocItem, error::ControlError, function::FUNCTION_TYPE, Freeze, FrozenStringValue,
+        Heap, Trace, Value, ValueError,
     },
 };
 
@@ -278,6 +278,11 @@ pub trait StarlarkValue<'v>: 'v + AnyLifetime<'v> + Debug + Display + Serialize
         write!(collector, "<{}...>", self.get_type()).unwrap()
     }
 
+    /// String used when printing call stack. `repr(self)` by default.
+    fn name_for_call_stack(&self, me: Value<'v>) -> String {
+        me.to_repr()
+    }
+
     /// Convert self to a boolean, as returned by the bool() function.
     /// The default implementation returns [`true`].
     fn to_bool(&self) -> bool {
@@ -522,8 +527,8 @@ pub trait StarlarkValue<'v>: 'v + AnyLifetime<'v> + Debug + Display + Serialize
     /// (1, 2, 3) + (2, 3) == (1, 2, 3, 2, 3)
     /// # "#);
     /// ```
-    fn add(&self, rhs: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
-        ValueError::unsupported_with(self, "+", rhs)
+    fn add(&self, _rhs: Value<'v>, _heap: &'v Heap) -> Option<anyhow::Result<Value<'v>>> {
+        None
     }
 
     /// Substract `other` from the current value.
@@ -621,7 +626,7 @@ pub trait StarlarkValue<'v>: 'v + AnyLifetime<'v> + Debug + Display + Serialize
     }
 
     /// Bitwise `&` operator.
-    fn bit_and(&self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
+    fn bit_and(&self, other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         ValueError::unsupported_with(self, "&", other)
     }
 
@@ -642,17 +647,22 @@ pub trait StarlarkValue<'v>: 'v + AnyLifetime<'v> + Debug + Display + Serialize
     }
 
     /// Bitwise `^` operator.
-    fn bit_xor(&self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
+    fn bit_xor(&self, other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         ValueError::unsupported_with(self, "^", other)
     }
 
+    /// Bitwise `~` operator.
+    fn bit_not(&self, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
+        ValueError::unsupported(self, "~")
+    }
+
     /// Bitwise `<<` operator.
-    fn left_shift(&self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
+    fn left_shift(&self, other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         ValueError::unsupported_with(self, "<<", other)
     }
 
     /// Bitwise `>>` operator.
-    fn right_shift(&self, other: Value<'v>) -> anyhow::Result<Value<'v>> {
+    fn right_shift(&self, other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>> {
         ValueError::unsupported_with(self, ">>", other)
     }
 
@@ -683,17 +693,12 @@ pub trait StarlarkValue<'v>: 'v + AnyLifetime<'v> + Debug + Display + Serialize
     }
 }
 
-erased_serde::serialize_trait_object!(<'v> StarlarkValue<'v>);
-
 /// Trait implemented by a value stored in arena which delegates
 /// it's operations to contained [`StarlarkValue`].
 pub(crate) trait StarlarkValueDyn<'v>: 'v + Serialize {
     // `AValue` is not a `StarlarkValue`, but a `Wrapper` type (or `BlackHole`).
     // `static_type_xxx_of_value` operations return `TypeId` of that `StarlarkValue`,
     // which is not the same of `TypeId` of `AValue` (because `AValue` is a wrapper).
-    fn static_type_id_of_value() -> TypeId
-    where
-        Self: Sized;
     fn static_type_of_value(&self) -> TypeId;
 
     fn as_debug(&self) -> &dyn Debug;
@@ -711,6 +716,7 @@ pub(crate) trait StarlarkValueDyn<'v>: 'v + Serialize {
     fn documentation(&self) -> Option<DocItem>;
     fn collect_repr(&self, _collector: &mut String);
     fn collect_repr_cycle(&self, _collector: &mut String);
+    fn name_for_call_stack(&self, me: Value<'v>) -> String;
     fn to_bool(&self) -> bool;
     fn to_int(&self) -> anyhow::Result<i32>;
     fn write_hash(&self, hasher: &mut StarlarkHasher) -> anyhow::Result<()>;
@@ -757,20 +763,19 @@ pub(crate) trait StarlarkValueDyn<'v>: 'v + Serialize {
     fn plus(&self, _heap: &'v Heap) -> anyhow::Result<Value<'v>>;
     fn minus(&self, _heap: &'v Heap) -> anyhow::Result<Value<'v>>;
     fn radd(&self, _lhs: Value<'v>, _heap: &'v Heap) -> Option<anyhow::Result<Value<'v>>>;
-    fn add(&self, _rhs: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>>;
+    fn add(&self, _rhs: Value<'v>, _heap: &'v Heap) -> Option<anyhow::Result<Value<'v>>>;
     fn sub(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>>;
     fn mul(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>>;
     fn percent(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>>;
     fn div(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>>;
     fn floor_div(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>>;
-    fn bit_and(&self, _other: Value<'v>) -> anyhow::Result<Value<'v>>;
+    fn bit_and(&self, _other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>>;
     fn bit_or(&self, _other: Value<'v>, _heap: &'v Heap) -> anyhow::Result<Value<'v>>;
-    fn bit_xor(&self, _other: Value<'v>) -> anyhow::Result<Value<'v>>;
-    fn left_shift(&self, _other: Value<'v>) -> anyhow::Result<Value<'v>>;
-    fn right_shift(&self, _other: Value<'v>) -> anyhow::Result<Value<'v>>;
+    fn bit_xor(&self, _other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>>;
+    fn bit_not(&self, heap: &'v Heap) -> anyhow::Result<Value<'v>>;
+    fn left_shift(&self, _other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>>;
+    fn right_shift(&self, _other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>>;
     fn export_as(&self, _variable_name: &str, _eval: &mut Evaluator<'v, '_>);
     fn set_at(&self, _index: Value<'v>, _new_value: Value<'v>) -> anyhow::Result<()>;
     fn set_attr(&self, _attribute: &str, _new_value: Value<'v>) -> anyhow::Result<()>;
 }
-
-erased_serde::serialize_trait_object!(<'v> StarlarkValueDyn<'v>);
