@@ -122,18 +122,28 @@ impl<'v> List<'v> {
         }
     }
 
-    pub(crate) fn from_value_mut(x: Value<'v>) -> anyhow::Result<Option<&'v Self>> {
-        if unlikely(x.unpack_frozen().is_some()) {
-            return Err(ValueError::CannotMutateImmutableValue.into());
-        }
-        let ptr = x.downcast_ref::<ListGen<List<'v>>>();
-        match ptr {
-            None => Ok(None),
-            Some(ptr) => {
-                ptr.0.check_can_mutate()?;
-                Ok(Some(&ptr.0))
+    #[inline]
+    pub(crate) fn from_value_mut(x: Value<'v>) -> anyhow::Result<&'v Self> {
+        #[derive(thiserror::Error, Debug)]
+        #[error("Value is not list, value type: `{}`", .0)]
+        struct NotListError(&'static str);
+
+        #[cold]
+        #[inline(never)]
+        fn error<'v>(x: Value<'v>) -> anyhow::Error {
+            if x.downcast_ref::<ListGen<FrozenList>>().is_some() {
+                ValueError::CannotMutateImmutableValue.into()
+            } else {
+                NotListError(x.get_type()).into()
             }
         }
+
+        if let Some(x) = x.downcast_ref::<ListGen<List<'v>>>() {
+            x.0.check_can_mutate()?;
+            Ok(&x.0)
+        } else {
+            Err(error(x))
+        }
     }
 
     pub(crate) unsafe fn from_value_unchecked_mut(x: Value<'v>) -> &'v Self {
@@ -546,12 +556,9 @@ where
         self.0.with_iterator(f)
     }
 
-    fn add(&self, other: Value<'v>, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
-        if let Some(other) = List::from_value(other) {
-            Ok(heap.alloc_list_concat(self.0.content(), other.content()))
-        } else {
-            ValueError::unsupported_with(self, "+", other)
-        }
+    fn add(&self, other: Value<'v>, heap: &'v Heap) -> Option<anyhow::Result<Value<'v>>> {
+        List::from_value(other)
+            .map(|other| Ok(heap.alloc_list_concat(self.0.content(), other.content())))
     }
 
     fn mul(&self, other: Value, heap: &'v Heap) -> anyhow::Result<Value<'v>> {
