@@ -17,6 +17,7 @@
 
 pub mod any;
 pub mod array;
+pub mod bigint;
 pub mod bool;
 pub mod dict;
 pub mod enumeration;
@@ -28,6 +29,7 @@ pub mod list;
 pub mod none;
 pub mod range;
 pub mod record;
+pub mod regex;
 pub mod string;
 pub mod structs;
 pub mod tuple;
