@@ -58,8 +58,9 @@ use crate::{
         docs,
         docs::DocItem,
         error::ValueError,
-        AllocValue, Freeze, FrozenValue, Heap, StarlarkValue, StringValue, StringValueLike, Trace,
-        UnpackValue, Value, ValueLike, ValueOf,
+        layout::typed::string::StringValueLike,
+        AllocValue, Freeze, FrozenValue, Heap, StarlarkValue, StringValue, Trace, UnpackValue,
+        Value, ValueLike, ValueOf,
     },
 };
 
@@ -199,11 +200,7 @@ where
             .fields
             .iter()
             .map(|(k, v)| {
-                let name = k
-                    .to_string_value()
-                    .unpack_starlark_str()
-                    .unpack()
-                    .to_owned();
+                let name = k.to_string_value().as_str().to_owned();
                 match v.to_value().documentation() {
                     Some(DocItem::Function(f)) => (name, docs::Member::Function(f)),
                     _ => (
